@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2019 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -18,9 +18,12 @@
 #define DGL_NANO_WIDGET_HPP_INCLUDED
 
 #include "Color.hpp"
+#include "OpenGL.hpp"
 #include "Widget.hpp"
 
-#define NANOVG_DEJAVU_SANS_TTF "__dpf_dejavusans_ttf__"
+#ifndef DGL_NO_SHARED_RESOURCES
+# define NANOVG_DEJAVU_SANS_TTF "__dpf_dejavusans_ttf__"
+#endif
 
 struct NVGcontext;
 struct NVGpaint;
@@ -844,10 +847,12 @@ public:
     */
     int textBreakLines(const char* string, const char* end, float breakRowWidth, TextRow& rows, int maxRows);
 
+#ifndef DGL_NO_SHARED_RESOURCES
    /**
       Load DPF's internal shared resources for this NanoVG class.
     */
     virtual void loadSharedResources();
+#endif
 
 private:
     NVGcontext* const fContext;
