@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2019 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -15,9 +15,12 @@
  */
 
 #include "../NanoVG.hpp"
-#include "Resources.hpp"
 #include "WidgetPrivateData.hpp"
 
+#ifndef DGL_NO_SHARED_RESOURCES
+# include "Resources.hpp"
+#endif
+
 // -----------------------------------------------------------------------
 
 #if defined(DISTRHO_OS_WINDOWS)
@@ -78,11 +81,8 @@ static NVGcontext* nvgCreateGL_helper(int flags)
 {
 #if defined(DISTRHO_OS_WINDOWS)
     static bool needsInit = true;
-    if (needsInit)
-    {
-        needsInit = false;
 # define DGL_EXT(PROC, func) \
-      func = (PROC) wglGetProcAddress ( #func ); \
+      if (needsInit) func = (PROC) wglGetProcAddress ( #func ); \
       DISTRHO_SAFE_ASSERT_RETURN(func != nullptr, nullptr);
 DGL_EXT(PFNGLACTIVETEXTUREPROC,            glActiveTexture)
 DGL_EXT(PFNGLATTACHSHADERPROC,             glAttachShader)
@@ -112,7 +112,7 @@ DGL_EXT(PFNGLUNIFORM4FVPROC,               glUniform4fv)
 DGL_EXT(PFNGLUSEPROGRAMPROC,               glUseProgram)
 DGL_EXT(PFNGLVERTEXATTRIBPOINTERPROC,      glVertexAttribPointer)
 # undef DGL_EXT
-    }
+    needsInit = false;
 #endif
     return nvgCreateGL(flags);
 }
@@ -121,6 +121,25 @@ DGL_EXT(PFNGLVERTEXATTRIBPOINTERPROC,      glVertexAttribPointer)
 
 START_NAMESPACE_DGL
 
+// -----------------------------------------------------------------------
+// DGL Color class conversion
+
+Color::Color(const NVGcolor& c) noexcept
+    : red(c.r), green(c.g), blue(c.b), alpha(c.a)
+{
+    fixBounds();
+}
+
+Color::operator NVGcolor() const noexcept
+{
+    NVGcolor nc;
+    nc.r = red;
+    nc.g = green;
+    nc.b = blue;
+    nc.a = alpha;
+    return nc;
+}
+
 // -----------------------------------------------------------------------
 // NanoImage
 
@@ -444,10 +463,8 @@ void NanoVG::translate(float x, float y)
 
 void NanoVG::rotate(float angle)
 {
-    if (fContext == nullptr) return;
-    DISTRHO_SAFE_ASSERT_RETURN(angle > 0.0f,);
-
-    nvgRotate(fContext, angle);
+    if (fContext != nullptr)
+        nvgRotate(fContext, angle);
 }
 
 void NanoVG::skewX(float angle)
@@ -469,8 +486,8 @@ void NanoVG::skewY(float angle)
 void NanoVG::scale(float x, float y)
 {
     if (fContext == nullptr) return;
-    DISTRHO_SAFE_ASSERT_RETURN(x > 0.0f,);
-    DISTRHO_SAFE_ASSERT_RETURN(y > 0.0f,);
+    DISTRHO_SAFE_ASSERT_RETURN(d_isNotZero(x),);
+    DISTRHO_SAFE_ASSERT_RETURN(d_isNotZero(y),);
 
     nvgScale(fContext, x, y);
 }
@@ -860,9 +877,9 @@ float NanoVG::textBounds(float x, float y, const char* string, const char* end,
     if (fContext == nullptr) return 0.0f;
     DISTRHO_SAFE_ASSERT_RETURN(string != nullptr && string[0] != '\0', 0.0f);
 
-    float b[4];
+    float b[4] = { 0.0f, 0.0f, 0.0f, 0.0f };
     const float ret = nvgTextBounds(fContext, x, y, string, end, b);
-    bounds = Rectangle<float>(b[0], b[1], b[2], b[3]);
+    bounds = Rectangle<float>(b[0], b[1], b[2] - b[0], b[3] - b[1]);
     return ret;
 }
 
@@ -895,8 +912,11 @@ int NanoVG::textBreakLines(const char* string, const char* end, float breakRowWi
     return 0;
 }
 
+#ifndef DGL_NO_SHARED_RESOURCES
 void NanoVG::loadSharedResources()
 {
+    if (fContext == nullptr) return;
+
     if (nvgFindFont(fContext, NANOVG_DEJAVU_SANS_TTF) >= 0)
         return;
 
@@ -904,6 +924,7 @@ void NanoVG::loadSharedResources()
 
     nvgCreateFontMem(fContext, NANOVG_DEJAVU_SANS_TTF, (const uchar*)dejavusans_ttf, dejavusans_ttf_size, 0);
 }
+#endif
 
 // -----------------------------------------------------------------------
 
@@ -972,8 +993,18 @@ END_NAMESPACE_DGL
 
 #undef final
 
+#if defined(__GNUC__) && (__GNUC__ >= 6)
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wmisleading-indentation"
+# pragma GCC diagnostic ignored "-Wshift-negative-value"
+#endif
+
 extern "C" {
 #include "nanovg/nanovg.c"
 }
 
+#if defined(__GNUC__) && (__GNUC__ >= 6)
+# pragma GCC diagnostic pop
+#endif
+
 // -----------------------------------------------------------------------
