@@ -1,6 +1,8 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2019 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2019 Jean Pierre Cimalando <jp-dev@inbox.ru>
+ * Copyright (C) 2019 Robin Gareus <robin@gareus.org>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -19,16 +21,33 @@
 
 #include "../Base.hpp"
 
-#undef PUGL_HAVE_CAIRO
-#undef PUGL_HAVE_GL
-#define PUGL_HAVE_GL 1
+#ifdef DGL_CAIRO
+# define PUGL_CAIRO
+# include "../Cairo.hpp"
+#endif
+#ifdef DGL_OPENGL
+# define PUGL_OPENGL
+# include "../OpenGL.hpp"
+#endif
 
 #include "pugl/pugl.h"
 
-#if defined(DISTRHO_OS_WINDOWS)
-# include "pugl/pugl_win.cpp"
+#if defined(__GNUC__) && (__GNUC__ >= 7)
+# pragma GCC diagnostic push
+# pragma GCC diagnostic ignored "-Wimplicit-fallthrough"
+#endif
+
+#if defined(DISTRHO_OS_HAIKU)
+# define DGL_DEBUG_EVENTS
+# include "pugl/pugl_haiku.cpp"
 #elif defined(DISTRHO_OS_MAC)
+# define PuglWindow     DISTRHO_JOIN_MACRO(PuglWindow,     DGL_NAMESPACE)
+# define PuglOpenGLView DISTRHO_JOIN_MACRO(PuglOpenGLView, DGL_NAMESPACE)
 # include "pugl/pugl_osx.m"
+#elif defined(DISTRHO_OS_WINDOWS)
+# include "pugl/pugl_win.cpp"
+# undef max
+# undef min
 #else
 # include <sys/types.h>
 # include <unistd.h>
@@ -37,6 +56,10 @@ extern "C" {
 }
 #endif
 
+#if defined(__GNUC__) && (__GNUC__ >= 7)
+# pragma GCC diagnostic pop
+#endif
+
 #include "ApplicationPrivateData.hpp"
 #include "WidgetPrivateData.hpp"
 #include "../StandaloneWindow.hpp"
@@ -48,7 +71,7 @@ extern "C" {
 #define FOR_EACH_WIDGET_INV(rit) \
   for (std::list<Widget*>::reverse_iterator rit = fWidgets.rbegin(); rit != fWidgets.rend(); ++rit)
 
-#ifdef DEBUG
+#if defined(DEBUG) && defined(DGL_DEBUG_EVENTS)
 # define DBG(msg)  std::fprintf(stderr, "%s", msg);
 # define DBGp(...) std::fprintf(stderr, __VA_ARGS__);
 # define DBGF      std::fflush(stderr);
@@ -74,15 +97,27 @@ struct Window::PrivateData {
           fUsingEmbed(false),
           fWidth(1),
           fHeight(1),
+          fScaling(1.0),
+          fAutoScaling(1.0),
           fTitle(nullptr),
           fWidgets(),
           fModal(),
-#if defined(DISTRHO_OS_WINDOWS)
-          hwnd(0)
+#if defined(DISTRHO_OS_HAIKU)
+          bApplication(nullptr),
+          bView(nullptr),
+          bWindow(nullptr)
 #elif defined(DISTRHO_OS_MAC)
           fNeedsIdle(true),
           mView(nullptr),
-          mWindow(nullptr)
+          mWindow(nullptr),
+          mParentWindow(nullptr)
+# ifndef DGL_FILE_BROWSER_DISABLED
+        , fOpenFilePanel(nullptr),
+          fFilePanelDelegate(nullptr)
+# endif
+#elif defined(DISTRHO_OS_WINDOWS)
+          hwnd(nullptr),
+          hwndParent(nullptr)
 #else
           xDisplay(nullptr),
           xWindow(0)
@@ -102,15 +137,27 @@ struct Window::PrivateData {
           fUsingEmbed(false),
           fWidth(1),
           fHeight(1),
+          fScaling(1.0),
+          fAutoScaling(1.0),
           fTitle(nullptr),
           fWidgets(),
           fModal(parent.pData),
-#if defined(DISTRHO_OS_WINDOWS)
-          hwnd(0)
+#if defined(DISTRHO_OS_HAIKU)
+          bApplication(nullptr),
+          bView(nullptr),
+          bWindow(nullptr)
 #elif defined(DISTRHO_OS_MAC)
           fNeedsIdle(false),
           mView(nullptr),
-          mWindow(nullptr)
+          mWindow(nullptr),
+          mParentWindow(nullptr)
+# ifndef DGL_FILE_BROWSER_DISABLED
+        , fOpenFilePanel(nullptr),
+          fFilePanelDelegate(nullptr)
+# endif
+#elif defined(DISTRHO_OS_WINDOWS)
+          hwnd(nullptr),
+          hwndParent(nullptr)
 #else
           xDisplay(nullptr),
           xWindow(0)
@@ -120,35 +167,51 @@ struct Window::PrivateData {
         init();
 
         const PuglInternals* const parentImpl(parent.pData->fView->impl);
-#if defined(DISTRHO_OS_WINDOWS)
+
+        // NOTE: almost a 1:1 copy of setTransientWinId()
+#if defined(DISTRHO_OS_HAIKU)
         // TODO
 #elif defined(DISTRHO_OS_MAC)
-        // TODO
-        //[parentImpl->window orderWindow:NSWindowBelow relativeTo:[[mView window] windowNumber]];
+        mParentWindow = parentImpl->window;
+#elif defined(DISTRHO_OS_WINDOWS)
+        hwndParent = parentImpl->hwnd;
+        SetWindowLongPtr(hwnd, GWLP_HWNDPARENT, (LONG_PTR)hwndParent);
 #else
         XSetTransientForHint(xDisplay, xWindow, parentImpl->win);
 #endif
     }
 
-    PrivateData(Application& app, Window* const self, const intptr_t parentId)
+    PrivateData(Application& app, Window* const self, const intptr_t parentId, const double scaling, const bool resizable)
         : fApp(app),
           fSelf(self),
           fView(puglInit()),
           fFirstInit(true),
           fVisible(parentId != 0),
-          fResizable(parentId == 0),
+          fResizable(resizable),
           fUsingEmbed(parentId != 0),
           fWidth(1),
           fHeight(1),
+          fScaling(scaling),
+          fAutoScaling(1.0),
           fTitle(nullptr),
           fWidgets(),
           fModal(),
-#if defined(DISTRHO_OS_WINDOWS)
-          hwnd(0)
+#if defined(DISTRHO_OS_HAIKU)
+          bApplication(nullptr),
+          bView(nullptr),
+          bWindow(nullptr)
 #elif defined(DISTRHO_OS_MAC)
           fNeedsIdle(parentId == 0),
           mView(nullptr),
-          mWindow(nullptr)
+          mWindow(nullptr),
+          mParentWindow(nullptr)
+# ifndef DGL_FILE_BROWSER_DISABLED
+        , fOpenFilePanel(nullptr),
+          fFilePanelDelegate(nullptr)
+# endif
+#elif defined(DISTRHO_OS_WINDOWS)
+          hwnd(nullptr),
+          hwndParent(nullptr)
 #else
           xDisplay(nullptr),
           xWindow(0)
@@ -183,7 +246,6 @@ struct Window::PrivateData {
             return;
         }
 
-        puglInitContextType(fView, PUGL_GL);
         puglInitUserResizable(fView, fResizable);
         puglInitWindowSize(fView, static_cast<int>(fWidth), static_cast<int>(fHeight));
 
@@ -196,16 +258,20 @@ struct Window::PrivateData {
         puglSetSpecialFunc(fView, onSpecialCallback);
         puglSetReshapeFunc(fView, onReshapeCallback);
         puglSetCloseFunc(fView, onCloseCallback);
+#ifndef DGL_FILE_BROWSER_DISABLED
         puglSetFileSelectedFunc(fView, fileBrowserSelectedCallback);
+#endif
 
         puglCreateWindow(fView, nullptr);
 
         PuglInternals* impl = fView->impl;
-#if defined(DISTRHO_OS_WINDOWS)
-        hwnd = impl->hwnd;
-        DISTRHO_SAFE_ASSERT(hwnd != 0);
+
+#if defined(DISTRHO_OS_HAIKU)
+        bApplication = impl->app;
+        bView        = impl->view;
+        bWindow      = impl->window;
 #elif defined(DISTRHO_OS_MAC)
-        mView   = impl->glview;
+        mView   = impl->view;
         mWindow = impl->window;
         DISTRHO_SAFE_ASSERT(mView != nullptr);
         if (fUsingEmbed) {
@@ -213,6 +279,9 @@ struct Window::PrivateData {
         } else {
             DISTRHO_SAFE_ASSERT(mWindow != nullptr);
         }
+#elif defined(DISTRHO_OS_WINDOWS)
+        hwnd = impl->hwnd;
+        DISTRHO_SAFE_ASSERT(hwnd != 0);
 #else
         xDisplay = impl->display;
         xWindow  = impl->win;
@@ -220,9 +289,19 @@ struct Window::PrivateData {
 
         if (! fUsingEmbed)
         {
-            pid_t pid = getpid();
-            Atom _nwp = XInternAtom(xDisplay, "_NET_WM_PID", True);
+            const pid_t pid = getpid();
+            const Atom _nwp = XInternAtom(xDisplay, "_NET_WM_PID", False);
             XChangeProperty(xDisplay, xWindow, _nwp, XA_CARDINAL, 32, PropModeReplace, (const uchar*)&pid, 1);
+
+            const Atom _wt = XInternAtom(xDisplay, "_NET_WM_WINDOW_TYPE", False);
+
+            // Setting the window to both dialog and normal will produce a decorated floating dialog
+            // Order is important: DIALOG needs to come before NORMAL
+            const Atom _wts[2] = {
+                XInternAtom(xDisplay, "_NET_WM_WINDOW_TYPE_DIALOG", False),
+                XInternAtom(xDisplay, "_NET_WM_WINDOW_TYPE_NORMAL", False)
+            };
+            XChangeProperty(xDisplay, xWindow, _wt, XA_ATOM, 32, PropModeReplace, (const uchar*)&_wts, 2);
         }
 #endif
         puglEnterContext(fView);
@@ -268,16 +347,33 @@ struct Window::PrivateData {
             fTitle = nullptr;
         }
 
-#if defined(DISTRHO_OS_WINDOWS)
-        hwnd = 0;
+#if defined(DISTRHO_OS_HAIKU)
+        bApplication = nullptr;
+        bView        = nullptr;
+        bWindow      = nullptr;
 #elif defined(DISTRHO_OS_MAC)
         mView   = nullptr;
         mWindow = nullptr;
+#elif defined(DISTRHO_OS_WINDOWS)
+        hwnd = 0;
 #else
         xDisplay = nullptr;
         xWindow  = 0;
 #endif
 
+#if defined(DISTRHO_OS_MAC) && !defined(DGL_FILE_BROWSER_DISABLED)
+        if (fOpenFilePanel)
+        {
+            [fOpenFilePanel release];
+            fOpenFilePanel = nullptr;
+        }
+        if (fFilePanelDelegate)
+        {
+            [fFilePanelDelegate release];
+            fFilePanelDelegate = nullptr;
+        }
+#endif
+
         DBG("Success!\n");
     }
 
@@ -330,22 +426,6 @@ struct Window::PrivateData {
         fModal.enabled = true;
         fModal.parent->fModal.childFocus = this;
 
-#ifdef DISTRHO_OS_WINDOWS
-        // Center this window
-        PuglInternals* const parentImpl = fModal.parent->fView->impl;
-
-        RECT curRect;
-        RECT parentRect;
-        GetWindowRect(hwnd, &curRect);
-        GetWindowRect(parentImpl->hwnd, &parentRect);
-
-        int x = parentRect.left+(parentRect.right-curRect.right)/2;
-        int y = parentRect.top +(parentRect.bottom-curRect.bottom)/2;
-
-        SetWindowPos(hwnd, 0, x, y, 0, 0, SWP_NOACTIVATE|SWP_NOOWNERZORDER|SWP_NOSIZE|SWP_NOZORDER);
-        UpdateWindow(hwnd);
-#endif
-
         fModal.parent->setVisible(true);
         setVisible(true);
 
@@ -363,10 +443,12 @@ struct Window::PrivateData {
 
             // the mouse position probably changed since the modal appeared,
             // so send a mouse motion event to the modal's parent window
-#if defined(DISTRHO_OS_WINDOWS)
+#if defined(DISTRHO_OS_HAIKU)
             // TODO
 #elif defined(DISTRHO_OS_MAC)
             // TODO
+#elif defined(DISTRHO_OS_WINDOWS)
+            // TODO
 #else
             int i, wx, wy;
             uint u;
@@ -384,17 +466,26 @@ struct Window::PrivateData {
     void focus()
     {
         DBG("Window focus\n");
-#if defined(DISTRHO_OS_WINDOWS)
+#if defined(DISTRHO_OS_HAIKU)
+        if (bWindow != nullptr)
+        {
+            if (bWindow->LockLooper())
+            {
+                bWindow->Activate(true);
+                bWindow->UnlockLooper();
+            }
+        }
+        else
+        {
+            bView->MakeFocus(true);
+        }
+#elif defined(DISTRHO_OS_MAC)
+        if (mWindow != nullptr)
+            [mWindow makeKeyWindow];
+#elif defined(DISTRHO_OS_WINDOWS)
         SetForegroundWindow(hwnd);
         SetActiveWindow(hwnd);
         SetFocus(hwnd);
-#elif defined(DISTRHO_OS_MAC)
-        if (mWindow != nullptr)
-        {
-            // TODO
-            //[NSApp activateIgnoringOtherApps:YES];
-            //[mWindow makeKeyAndOrderFront:mWindow];
-        }
 #else
         XRaiseWindow(xDisplay, xWindow);
         XSetInputFocus(xDisplay, xWindow, RevertToPointerRoot, CurrentTime);
@@ -424,28 +515,90 @@ struct Window::PrivateData {
         if (yesNo && fFirstInit)
             setSize(fWidth, fHeight, true);
 
-#if defined(DISTRHO_OS_WINDOWS)
-        if (yesNo)
-            ShowWindow(hwnd, fFirstInit ? SW_SHOWNORMAL : SW_RESTORE);
-        else
-            ShowWindow(hwnd, SW_HIDE);
+#if defined(DISTRHO_OS_HAIKU)
+        if (bWindow != nullptr)
+        {
+            if (bWindow->LockLooper())
+            {
+                if (yesNo)
+                    bWindow->Show();
+                else
+                    bWindow->Hide();
 
-        UpdateWindow(hwnd);
+                // TODO use flush?
+                bWindow->Sync();
+                bWindow->UnlockLooper();
+            }
+        }
+        else
+        {
+            if (yesNo)
+                bView->Show();
+            else
+                bView->Hide();
+        }
 #elif defined(DISTRHO_OS_MAC)
         if (yesNo)
         {
             if (mWindow != nullptr)
+            {
+                if (mParentWindow != nullptr)
+                    [mParentWindow addChildWindow:mWindow
+                                          ordered:NSWindowAbove];
+
                 [mWindow setIsVisible:YES];
+            }
             else
+            {
                 [mView setHidden:NO];
+            }
         }
         else
         {
             if (mWindow != nullptr)
+            {
+                if (mParentWindow != nullptr)
+                    [mParentWindow removeChildWindow:mWindow];
+
                 [mWindow setIsVisible:NO];
+            }
             else
+            {
                 [mView setHidden:YES];
+            }
         }
+#elif defined(DISTRHO_OS_WINDOWS)
+        if (yesNo)
+        {
+            if (fFirstInit)
+            {
+                RECT rectChild, rectParent;
+
+                if (hwndParent != nullptr &&
+                    GetWindowRect(hwnd, &rectChild) &&
+                    GetWindowRect(hwndParent, &rectParent))
+                {
+                    SetWindowPos(hwnd, hwndParent,
+                                 rectParent.left + (rectChild.right-rectChild.left)/2,
+                                 rectParent.top + (rectChild.bottom-rectChild.top)/2,
+                                 0, 0, SWP_SHOWWINDOW|SWP_NOSIZE);
+                }
+                else
+                {
+                    ShowWindow(hwnd, SW_SHOWNORMAL);
+                }
+            }
+            else
+            {
+                ShowWindow(hwnd, SW_RESTORE);
+            }
+        }
+        else
+        {
+            ShowWindow(hwnd, SW_HIDE);
+        }
+
+        UpdateWindow(hwnd);
 #else
         if (yesNo)
             XMapRaised(xDisplay, xWindow);
@@ -485,14 +638,20 @@ struct Window::PrivateData {
         DBG("Window setResizable called\n");
 
         fResizable = yesNo;
+        fView->user_resizable = yesNo;
 
-#if defined(DISTRHO_OS_WINDOWS)
+#if defined(DISTRHO_OS_HAIKU)
+        // TODO
+        // B_NO_BORDER
+        // B_TITLED_WINDOW_LOOK
+        // bWindow->SetFlags();
+#elif defined(DISTRHO_OS_MAC)
+        const uint flags = yesNo ? (NSViewWidthSizable|NSViewHeightSizable) : 0x0;
+        [mView setAutoresizingMask:flags];
+#elif defined(DISTRHO_OS_WINDOWS)
         const int winFlags = fResizable ? GetWindowLong(hwnd, GWL_STYLE) |  WS_SIZEBOX
                                         : GetWindowLong(hwnd, GWL_STYLE) & ~WS_SIZEBOX;
         SetWindowLong(hwnd, GWL_STYLE, winFlags);
-#elif defined(DISTRHO_OS_MAC)
-        const uint flags(yesNo ? (NSViewWidthSizable|NSViewHeightSizable) : 0x0);
-        [mView setAutoresizingMask:flags];
 #endif
 
         setSize(fWidth, fHeight, true);
@@ -500,6 +659,18 @@ struct Window::PrivateData {
 
     // -------------------------------------------------------------------
 
+    void setGeometryConstraints(uint width, uint height, bool aspect)
+    {
+        // Did you forget to set DISTRHO_UI_USER_RESIZABLE ?
+        DISTRHO_SAFE_ASSERT_RETURN(fResizable,);
+
+        fView->min_width  = width;
+        fView->min_height = height;
+        puglUpdateGeometryConstraints(fView, width, height, aspect);
+    }
+
+    // -------------------------------------------------------------------
+
     void setSize(uint width, uint height, const bool forced = false)
     {
         if (width <= 1 || height <= 1)
@@ -519,16 +690,20 @@ struct Window::PrivateData {
 
         DBGp("Window setSize called %s, size %i %i, resizable %s\n", forced ? "(forced)" : "(not forced)", width, height, fResizable?"true":"false");
 
-#if defined(DISTRHO_OS_WINDOWS)
-        const int winFlags = WS_POPUPWINDOW | WS_CAPTION | (fResizable ? WS_SIZEBOX : 0x0);
-        RECT wr = { 0, 0, static_cast<long>(width), static_cast<long>(height) };
-        AdjustWindowRectEx(&wr, fUsingEmbed ? WS_CHILD : winFlags, FALSE, WS_EX_TOPMOST);
+#if defined(DISTRHO_OS_HAIKU)
+        bView->ResizeTo(width, height);
 
-        SetWindowPos(hwnd, 0, 0, 0, wr.right-wr.left, wr.bottom-wr.top,
-                     SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOZORDER);
+        if (bWindow != nullptr && bWindow->LockLooper())
+        {
+            bWindow->MoveTo(50, 50);
+            bWindow->ResizeTo(width, height);
 
-        if (! forced)
-            UpdateWindow(hwnd);
+            if (! forced)
+                bWindow->Flush();
+
+            bWindow->UnlockLooper();
+        }
+        // TODO resizable
 #elif defined(DISTRHO_OS_MAC)
         [mView setFrame:NSMakeRect(0, 0, width, height)];
 
@@ -550,8 +725,17 @@ struct Window::PrivateData {
                 [[mWindow standardWindowButton:NSWindowZoomButton] setHidden:YES];
             }
         }
+#elif defined(DISTRHO_OS_WINDOWS)
+        const int winFlags = WS_POPUPWINDOW | WS_CAPTION | (fResizable ? WS_SIZEBOX : 0x0);
+        RECT wr = { 0, 0, static_cast<LONG>(width), static_cast<LONG>(height) };
+        AdjustWindowRectEx(&wr, fUsingEmbed ? WS_CHILD : winFlags, FALSE, WS_EX_TOPMOST);
+
+        SetWindowPos(hwnd, 0, 0, 0, wr.right-wr.left, wr.bottom-wr.top,
+                     SWP_NOACTIVATE|SWP_NOMOVE|SWP_NOOWNERZORDER|SWP_NOZORDER);
+
+        if (! forced)
+            UpdateWindow(hwnd);
 #else
-        XResizeWindow(xDisplay, xWindow, width, height);
 
         if (! fResizable)
         {
@@ -566,9 +750,11 @@ struct Window::PrivateData {
             sizeHints.max_width  = static_cast<int>(width);
             sizeHints.max_height = static_cast<int>(height);
 
-            XSetNormalHints(xDisplay, xWindow, &sizeHints);
+            XSetWMNormalHints(xDisplay, xWindow, &sizeHints);
         }
 
+        XResizeWindow(xDisplay, xWindow, width, height);
+
         if (! forced)
             XFlush(xDisplay);
 #endif
@@ -594,8 +780,12 @@ struct Window::PrivateData {
 
         fTitle = strdup(title);
 
-#if defined(DISTRHO_OS_WINDOWS)
-        SetWindowTextA(hwnd, title);
+#if defined(DISTRHO_OS_HAIKU)
+        if (bWindow != nullptr&& bWindow->LockLooper())
+        {
+            bWindow->SetTitle(title);
+            bWindow->UnlockLooper();
+        }
 #elif defined(DISTRHO_OS_MAC)
         if (mWindow != nullptr)
         {
@@ -606,17 +796,31 @@ struct Window::PrivateData {
 
             [mWindow setTitle:titleString];
         }
+#elif defined(DISTRHO_OS_WINDOWS)
+        SetWindowTextA(hwnd, title);
 #else
         XStoreName(xDisplay, xWindow, title);
+        Atom netWmName = XInternAtom(xDisplay, "_NET_WM_NAME", False);
+        Atom utf8String = XInternAtom(xDisplay, "UTF8_STRING", False);
+        XChangeProperty(xDisplay, xWindow, netWmName, utf8String, 8, PropModeReplace, (unsigned char *)title, strlen(title));
 #endif
     }
 
     void setTransientWinId(const uintptr_t winId)
     {
-#if defined(DISTRHO_OS_WINDOWS)
+        DISTRHO_SAFE_ASSERT_RETURN(winId != 0,);
+
+#if defined(DISTRHO_OS_HAIKU)
         // TODO
 #elif defined(DISTRHO_OS_MAC)
-        // TODO
+        NSWindow* const parentWindow = [NSApp windowWithWindowNumber:winId];
+        DISTRHO_SAFE_ASSERT_RETURN(parentWindow != nullptr,);
+
+        [parentWindow addChildWindow:mWindow
+                             ordered:NSWindowAbove];
+#elif defined(DISTRHO_OS_WINDOWS)
+        hwndParent = (HWND)winId;
+        SetWindowLongPtr(hwnd, GWLP_HWNDPARENT, (LONG_PTR)winId);
 #else
         XSetTransientForHint(xDisplay, xWindow, static_cast< ::Window>(winId));
 #endif
@@ -624,6 +828,32 @@ struct Window::PrivateData {
 
     // -------------------------------------------------------------------
 
+    double getScaling() const noexcept
+    {
+        return fScaling;
+    }
+
+    void setAutoScaling(const double scaling) noexcept
+    {
+        DISTRHO_SAFE_ASSERT_RETURN(scaling > 0.0,);
+
+        fAutoScaling = scaling;
+    }
+
+    // -------------------------------------------------------------------
+
+    bool getIgnoringKeyRepeat() const noexcept
+    {
+        return fView->ignoreKeyRepeat;
+    }
+
+    void setIgnoringKeyRepeat(bool ignore) noexcept
+    {
+        puglIgnoreKeyRepeat(fView, ignore);
+    }
+
+    // -------------------------------------------------------------------
+
     void addWidget(Widget* const widget)
     {
         fWidgets.push_back(widget);
@@ -638,6 +868,15 @@ struct Window::PrivateData {
     {
         puglProcessEvents(fView);
 
+#ifdef DISTRHO_OS_HAIKU
+        if (bApplication != nullptr)
+        {
+            // bApplication->Lock();
+            // bApplication->Loop();
+            // bApplication->Unlock();
+        }
+#endif
+
 #ifdef DISTRHO_OS_MAC
         if (fNeedsIdle)
         {
@@ -662,6 +901,15 @@ struct Window::PrivateData {
         }
 #endif
 
+#if defined(DISTRHO_OS_WINDOWS) && !defined(DGL_FILE_BROWSER_DISABLED)
+        if (fSelectedFile.isNotEmpty())
+        {
+            char* const buffer = fSelectedFile.getAndReleaseBuffer();
+            fView->fileSelectedFunc(fView, buffer);
+            std::free(buffer);
+        }
+#endif
+
         if (fModal.enabled && fModal.parent != nullptr)
             fModal.parent->idle();
     }
@@ -675,7 +923,7 @@ struct Window::PrivateData {
         FOR_EACH_WIDGET(it)
         {
             Widget* const widget(*it);
-            widget->pData->display(fWidth, fHeight);
+            widget->pData->display(fWidth, fHeight, fAutoScaling, false);
         }
 
         fSelf->onDisplayAfter();
@@ -735,7 +983,7 @@ struct Window::PrivateData {
         return 1;
     }
 
-    void onPuglMouse(const int button, const bool press, const int x, const int y)
+    void onPuglMouse(const int button, const bool press, int x, int y)
     {
         DBGp("PUGL: onMouse : %i %i %i %i\n", button, press, x, y);
 
@@ -745,6 +993,9 @@ struct Window::PrivateData {
         if (fModal.childFocus != nullptr)
             return fModal.childFocus->focus();
 
+        x /= fAutoScaling;
+        y /= fAutoScaling;
+
         Widget::MouseEvent ev;
         ev.button = button;
         ev.press  = press;
@@ -762,13 +1013,16 @@ struct Window::PrivateData {
         }
     }
 
-    void onPuglMotion(const int x, const int y)
+    void onPuglMotion(int x, int y)
     {
-        DBGp("PUGL: onMotion : %i %i\n", x, y);
+        // DBGp("PUGL: onMotion : %i %i\n", x, y);
 
         if (fModal.childFocus != nullptr)
             return;
 
+        x /= fAutoScaling;
+        y /= fAutoScaling;
+
         Widget::MotionEvent ev;
         ev.mod  = static_cast<Modifier>(puglGetModifiers(fView));
         ev.time = puglGetEventTimestamp(fView);
@@ -784,13 +1038,18 @@ struct Window::PrivateData {
         }
     }
 
-    void onPuglScroll(const int x, const int y, const float dx, const float dy)
+    void onPuglScroll(int x, int y, float dx, float dy)
     {
         DBGp("PUGL: onScroll : %i %i %f %f\n", x, y, dx, dy);
 
         if (fModal.childFocus != nullptr)
             return;
 
+        x /= fAutoScaling;
+        y /= fAutoScaling;
+        dx /= fAutoScaling;
+        dy /= fAutoScaling;
+
         Widget::ScrollEvent ev;
         ev.delta = Point<float>(dx, dy);
         ev.mod   = static_cast<Modifier>(puglGetModifiers(fView));
@@ -927,11 +1186,45 @@ struct Window::PrivateData {
         return false;
     }
 
+#if defined(DISTRHO_OS_MAC) && !defined(DGL_FILE_BROWSER_DISABLED)
+    static void openPanelDidEnd(NSOpenPanel* panel, int returnCode, void *userData)
+    {
+        PrivateData* pData = (PrivateData*)userData;
+
+        if (returnCode == NSOKButton)
+        {
+            NSArray* urls = [panel URLs];
+            NSURL* fileUrl = nullptr;
+
+            for (NSUInteger i = 0, n = [urls count]; i < n && !fileUrl; ++i)
+            {
+                NSURL* url = (NSURL*)[urls objectAtIndex:i];
+                if ([url isFileURL])
+                    fileUrl = url;
+            }
+
+            if (fileUrl)
+            {
+                PuglView* view = pData->fView;
+                if (view->fileSelectedFunc)
+                {
+                    const char* fileName = [fileUrl.path UTF8String];
+                    view->fileSelectedFunc(view, fileName);
+                }
+            }
+        }
+
+        [pData->fOpenFilePanel release];
+        pData->fOpenFilePanel = nullptr;
+    }
+#endif
+
     // -------------------------------------------------------------------
 
-    Application& fApp;
-    Window*      fSelf;
-    PuglView*    fView;
+    Application&    fApp;
+    Window*         fSelf;
+    GraphicsContext fContext;
+    PuglView*       fView;
 
     bool fFirstInit;
     bool fVisible;
@@ -939,6 +1232,8 @@ struct Window::PrivateData {
     bool fUsingEmbed;
     uint fWidth;
     uint fHeight;
+    double fScaling;
+    double fAutoScaling;
     char* fTitle;
     std::list<Widget*> fWidgets;
 
@@ -966,12 +1261,25 @@ struct Window::PrivateData {
         DISTRHO_DECLARE_NON_COPY_STRUCT(Modal)
     } fModal;
 
-#if defined(DISTRHO_OS_WINDOWS)
-    HWND     hwnd;
+#if defined(DISTRHO_OS_HAIKU)
+    BApplication* bApplication;
+    BView*        bView;
+    BWindow*      bWindow;
 #elif defined(DISTRHO_OS_MAC)
     bool            fNeedsIdle;
-    PuglOpenGLView* mView;
+    NSView<PuglGenericView>* mView;
     id              mWindow;
+    id              mParentWindow;
+# ifndef DGL_FILE_BROWSER_DISABLED
+    NSOpenPanel*    fOpenFilePanel;
+    id              fFilePanelDelegate;
+# endif
+#elif defined(DISTRHO_OS_WINDOWS)
+    HWND hwnd;
+    HWND hwndParent;
+# ifndef DGL_FILE_BROWSER_DISABLED
+    String fSelectedFile;
+# endif
 #else
     Display* xDisplay;
     ::Window xWindow;
@@ -1022,10 +1330,12 @@ struct Window::PrivateData {
         handlePtr->onPuglClose();
     }
 
+#ifndef DGL_FILE_BROWSER_DISABLED
     static void fileBrowserSelectedCallback(PuglView* view, const char* filename)
     {
         handlePtr->fSelf->fileBrowserSelected(filename);
     }
+#endif
 
     #undef handlePtr
 
@@ -1041,8 +1351,8 @@ Window::Window(Application& app)
 Window::Window(Application& app, Window& parent)
     : pData(new PrivateData(app, this, parent)) {}
 
-Window::Window(Application& app, intptr_t parentId)
-    : pData(new PrivateData(app, this, parentId)) {}
+Window::Window(Application& app, intptr_t parentId, double scaling, bool resizable)
+    : pData(new PrivateData(app, this, parentId, scaling, resizable)) {}
 
 Window::~Window()
 {
@@ -1085,9 +1395,24 @@ void Window::repaint() noexcept
 //     (void)name;
 // }
 
+#ifndef DGL_FILE_BROWSER_DISABLED
+
+#ifdef DISTRHO_OS_MAC
+END_NAMESPACE_DGL
+@interface FilePanelDelegate : NSObject
+{
+    void (*fCallback)(NSOpenPanel*, int, void*);
+    void* fUserData;
+}
+-(id)initWithCallback:(void(*)(NSOpenPanel*, int, void*))callback userData:(void*)userData;
+-(void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo;
+@end
+START_NAMESPACE_DGL
+#endif
+
 bool Window::openFileBrowser(const FileBrowserOptions& options)
 {
-#ifdef SOFD_HAVE_X11
+# ifdef SOFD_HAVE_X11
     using DISTRHO_NAMESPACE::String;
 
     // --------------------------------------------------------------------------
@@ -1098,6 +1423,7 @@ bool Window::openFileBrowser(const FileBrowserOptions& options)
 
     String startDir(options.startDir);
 
+#  ifdef DISTRHO_OS_LINUX
     if (startDir.isEmpty())
     {
         if (char* const dir_name = get_current_dir_name())
@@ -1106,6 +1432,7 @@ bool Window::openFileBrowser(const FileBrowserOptions& options)
             std::free(dir_name);
         }
     }
+#  endif
 
     DISTRHO_SAFE_ASSERT_RETURN(startDir.isNotEmpty(), false);
 
@@ -1145,10 +1472,127 @@ bool Window::openFileBrowser(const FileBrowserOptions& options)
     // show
 
     return (x_fib_show(pData->xDisplay, pData->xWindow, /*options.width*/0, /*options.height*/0) == 0);
-#else
+# elif defined(DISTRHO_OS_WINDOWS)
+    // the old and compatible dialog API
+    OPENFILENAMEW ofn;
+    memset(&ofn, 0, sizeof(ofn));
+
+    ofn.lStructSize = sizeof(ofn);
+    ofn.hwndOwner = pData->hwnd;
+
+    // set initial directory in UTF-16 coding
+    std::vector<WCHAR> startDirW;
+    if (options.startDir)
+    {
+        startDirW.resize(strlen(options.startDir) + 1);
+        if (MultiByteToWideChar(CP_UTF8, 0, options.startDir, -1, startDirW.data(), startDirW.size()))
+            ofn.lpstrInitialDir = startDirW.data();
+    }
+
+    // set title in UTF-16 coding
+    std::vector<WCHAR> titleW;
+    if (options.title)
+    {
+        titleW.resize(strlen(options.title) + 1);
+        if (MultiByteToWideChar(CP_UTF8, 0, options.title, -1, titleW.data(), titleW.size()))
+            ofn.lpstrTitle = titleW.data();
+    }
+
+    // prepare a buffer to receive the result
+    std::vector<WCHAR> fileNameW(32768); // the Unicode maximum
+    ofn.lpstrFile = fileNameW.data();
+    ofn.nMaxFile = (DWORD)fileNameW.size();
+
+    // TODO synchronous only, can't do better with WinAPI native dialogs.
+    // threading might work, if someone is motivated to risk it.
+    if (GetOpenFileNameW(&ofn))
+    {
+        // back to UTF-8
+        std::vector<char> fileNameA(4 * 32768);
+        if (WideCharToMultiByte(CP_UTF8, 0, fileNameW.data(), -1, fileNameA.data(), (int)fileNameA.size(), nullptr, nullptr))
+        {
+            // handle it during the next idle cycle (fake async)
+            pData->fSelectedFile = fileNameA.data();
+        }
+    }
+
+    return true;
+# elif defined(DISTRHO_OS_MAC)
+    if (pData->fOpenFilePanel) // permit one dialog at most
+    {
+        [pData->fOpenFilePanel makeKeyAndOrderFront:nil];
+        return false;
+    }
+
+    NSOpenPanel* panel = [NSOpenPanel openPanel];
+    pData->fOpenFilePanel = [panel retain];
+
+    [panel setCanChooseFiles:YES];
+    [panel setCanChooseDirectories:NO];
+    [panel setAllowsMultipleSelection:NO];
+
+    if (options.startDir)
+        [panel setDirectory:[NSString stringWithUTF8String:options.startDir]];
+
+    if (options.title)
+    {
+        NSString* titleString = [[NSString alloc]
+            initWithBytes:options.title
+                   length:strlen(options.title)
+                 encoding:NSUTF8StringEncoding];
+        [panel setTitle:titleString];
+    }
+
+    id delegate = pData->fFilePanelDelegate;
+    if (!delegate)
+    {
+        delegate = [[FilePanelDelegate alloc] initWithCallback:&PrivateData::openPanelDidEnd
+                                                      userData:pData];
+        pData->fFilePanelDelegate = [delegate retain];
+    }
+
+    [panel beginSheetForDirectory:nullptr
+                             file:nullptr
+                   modalForWindow:nullptr
+                    modalDelegate:delegate
+                   didEndSelector:@selector(openPanelDidEnd:returnCode:contextInfo:)
+                      contextInfo:nullptr];
+
+    return true;
+# else
     // not implemented
     return false;
+
+    // unused
+    (void)options;
+# endif
+}
+
+#ifdef DISTRHO_OS_MAC
+END_NAMESPACE_DGL
+@implementation FilePanelDelegate
+-(id)initWithCallback:(void(*)(NSOpenPanel*, int, void*))callback userData:(void *)userData
+{
+    [super init];
+    self->fCallback = callback;
+    self->fUserData = userData;
+    return self;
+}
+
+-(void)openPanelDidEnd:(NSOpenPanel *)sheet returnCode:(int)returnCode contextInfo:(void *)contextInfo
+{
+    self->fCallback(sheet, returnCode, self->fUserData);
+    (void)contextInfo;
+}
+@end
+START_NAMESPACE_DGL
 #endif
+
+#endif // !defined(DGL_FILE_BROWSER_DISABLED)
+
+bool Window::isEmbed() const noexcept
+{
+    return pData->fUsingEmbed;
 }
 
 bool Window::isVisible() const noexcept
@@ -1171,6 +1615,11 @@ void Window::setResizable(bool yesNo)
     pData->setResizable(yesNo);
 }
 
+void Window::setGeometryConstraints(uint width, uint height, bool aspect)
+{
+    pData->setGeometryConstraints(width, height, aspect);
+}
+
 uint Window::getWidth() const noexcept
 {
     return pData->fWidth;
@@ -1211,6 +1660,21 @@ void Window::setTransientWinId(uintptr_t winId)
     pData->setTransientWinId(winId);
 }
 
+double Window::getScaling() const noexcept
+{
+    return pData->getScaling();
+}
+
+bool Window::getIgnoringKeyRepeat() const noexcept
+{
+    return pData->getIgnoringKeyRepeat();
+}
+
+void Window::setIgnoringKeyRepeat(bool ignore) noexcept
+{
+    pData->setIgnoringKeyRepeat(ignore);
+}
+
 Application& Window::getApp() const noexcept
 {
     return pData->fApp;
@@ -1221,6 +1685,20 @@ intptr_t Window::getWindowId() const noexcept
     return puglGetNativeWindow(pData->fView);
 }
 
+const GraphicsContext& Window::getGraphicsContext() const noexcept
+{
+    GraphicsContext& context = pData->fContext;
+#ifdef DGL_CAIRO
+    context.cairo = (cairo_t*)puglGetContext(pData->fView);
+#endif
+    return context;
+}
+
+void Window::_setAutoScaling(double scaling) noexcept
+{
+    pData->setAutoScaling(scaling);
+}
+
 void Window::_addWidget(Widget* const widget)
 {
     pData->addWidget(widget);
@@ -1256,8 +1734,10 @@ void Window::removeIdleCallback(IdleCallback* const callback)
 
 void Window::onDisplayBefore()
 {
+#ifdef DGL_OPENGL
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
     glLoadIdentity();
+#endif
 }
 
 void Window::onDisplayAfter()
@@ -1266,6 +1746,7 @@ void Window::onDisplayAfter()
 
 void Window::onReshape(uint width, uint height)
 {
+#ifdef DGL_OPENGL
     glEnable(GL_BLEND);
     glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glMatrixMode(GL_PROJECTION);
@@ -1274,15 +1755,18 @@ void Window::onReshape(uint width, uint height)
     glViewport(0, 0, static_cast<GLsizei>(width), static_cast<GLsizei>(height));
     glMatrixMode(GL_MODELVIEW);
     glLoadIdentity();
+#endif
 }
 
 void Window::onClose()
 {
 }
 
+#ifndef DGL_FILE_BROWSER_DISABLED
 void Window::fileBrowserSelected(const char*)
 {
 }
+#endif
 
 bool Window::handlePluginKeyboard(const bool press, const uint key)
 {
