@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2017 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2019 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -70,6 +70,142 @@
 #define DISTRHO_LV2_USE_EVENTS_IN  (DISTRHO_PLUGIN_WANT_MIDI_INPUT || DISTRHO_PLUGIN_WANT_TIMEPOS || (DISTRHO_PLUGIN_WANT_STATE && DISTRHO_PLUGIN_HAS_UI))
 #define DISTRHO_LV2_USE_EVENTS_OUT (DISTRHO_PLUGIN_WANT_MIDI_OUTPUT || (DISTRHO_PLUGIN_WANT_STATE && DISTRHO_PLUGIN_HAS_UI))
 
+// -----------------------------------------------------------------------
+static const char* const lv2ManifestPluginExtensionData[] =
+{
+    "opts:interface",
+#if DISTRHO_PLUGIN_WANT_STATE
+    LV2_STATE__interface,
+    LV2_WORKER__interface,
+#endif
+#if DISTRHO_PLUGIN_WANT_PROGRAMS
+    LV2_PROGRAMS__Interface,
+#endif
+#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
+    MOD_LICENSE__interface,
+#endif
+    nullptr
+};
+
+static const char* const lv2ManifestPluginOptionalFeatures[] =
+{
+#if DISTRHO_PLUGIN_IS_RT_SAFE
+    LV2_CORE__hardRTCapable,
+#endif
+    LV2_BUF_SIZE__boundedBlockLength,
+    nullptr
+};
+
+static const char* const lv2ManifestPluginRequiredFeatures[] =
+{
+    "opts:options",
+    LV2_URID__map,
+#if DISTRHO_PLUGIN_WANT_STATE
+    LV2_WORKER__schedule,
+#endif
+#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
+    MOD_LICENSE__feature,
+#endif
+    nullptr
+};
+
+static const char* const lv2ManifestPluginSupportedOptions[] =
+{
+    LV2_BUF_SIZE__nominalBlockLength,
+    LV2_BUF_SIZE__maxBlockLength,
+    LV2_PARAMETERS__sampleRate,
+    nullptr
+};
+
+#if DISTRHO_PLUGIN_HAS_UI
+static const char* const lv2ManifestUiExtensionData[] =
+{
+    "opts:interface",
+    "ui:idleInterface",
+    "ui:showInterface",
+    "ui:resize",
+#if DISTRHO_PLUGIN_WANT_PROGRAMS
+    LV2_PROGRAMS__UIInterface,
+#endif
+    nullptr
+};
+
+static const char* const lv2ManifestUiOptionalFeatures[] =
+{
+#if DISTRHO_PLUGIN_HAS_EMBED_UI
+# if !DISTRHO_UI_USER_RESIZABLE
+    "ui:noUserResize",
+# endif
+    "ui:parent",
+    "ui:resize",
+    "ui:touch",
+#endif
+    nullptr
+};
+
+static const char* const lv2ManifestUiRequiredFeatures[] =
+{
+    "opts:options",
+    "ui:idleInterface",
+#if DISTRHO_PLUGIN_WANT_DIRECT_ACCESS
+    LV2_DATA_ACCESS_URI,
+    LV2_INSTANCE_ACCESS_URI,
+#endif
+    LV2_URID__map,
+    nullptr
+};
+
+static const char* const lv2ManifestUiSupportedOptions[] =
+{
+    LV2_PARAMETERS__sampleRate,
+    nullptr
+};
+#endif // DISTRHO_PLUGIN_HAS_UI
+
+static void addAttribute(DISTRHO_NAMESPACE::String& text,
+                         const char* const attribute,
+                         const char* const values[],
+                         const uint indent,
+                         const bool endInDot = false)
+{
+    if (values[0] == nullptr)
+    {
+        if (endInDot)
+        {
+            bool found;
+            const size_t index = text.rfind(';', &found);
+            if (found) text[index] = '.';
+        }
+        return;
+    }
+
+    const size_t attributeLength = std::strlen(attribute);
+
+    for (uint i = 0; values[i] != nullptr; ++i)
+    {
+        for (uint j = 0; j < indent; ++j)
+            text += " ";
+
+        if (i == 0)
+        {
+            text += attribute;
+        }
+        else
+        {
+            for (uint j = 0; j < attributeLength; ++j)
+                text += " ";
+        }
+
+        text += " ";
+
+        const bool isUrl = std::strstr(values[i], "://") != nullptr || std::strncmp(values[i], "urn:", 4) == 0;
+        if (isUrl) text += "<";
+        text += values[i];
+        if (isUrl) text += ">";
+        text += values[i + 1] ? " ,\n" : (endInDot ? " .\n\n" : " ;\n\n");
+    }
+}
+
 // -----------------------------------------------------------------------
 
 DISTRHO_PLUGIN_EXPORT
@@ -80,7 +216,7 @@ void lv2_generate_ttl(const char* const basename)
     // Dummy plugin to get data from
     d_lastBufferSize = 512;
     d_lastSampleRate = 44100.0;
-    PluginExporter plugin;
+    PluginExporter plugin(nullptr, nullptr);
     d_lastBufferSize = 0;
     d_lastSampleRate = 0.0;
 
@@ -105,6 +241,9 @@ void lv2_generate_ttl(const char* const basename)
         String manifestString;
         manifestString += "@prefix lv2:  <" LV2_CORE_PREFIX "> .\n";
         manifestString += "@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .\n";
+#if DISTRHO_PLUGIN_HAS_UI && DISTRHO_PLUGIN_WANT_DIRECT_ACCESS
+        manifestString += "@prefix opts: <" LV2_OPTIONS_PREFIX "> .\n";
+#endif
 #if DISTRHO_PLUGIN_WANT_PROGRAMS
         manifestString += "@prefix pset: <" LV2_PRESETS_PREFIX "> .\n";
 #endif
@@ -129,28 +268,13 @@ void lv2_generate_ttl(const char* const basename)
         manifestString += "    a ui:" DISTRHO_LV2_UI_TYPE " ;\n";
         manifestString += "    ui:binary <" + pluginUI + "." DISTRHO_DLL_EXTENSION "> ;\n";
 # if DISTRHO_PLUGIN_WANT_DIRECT_ACCESS
-        manifestString += "\n";
-        manifestString += "    lv2:extensionData ui:idleInterface ,\n";
-#  if DISTRHO_PLUGIN_WANT_PROGRAMS
-        manifestString += "                      ui:showInterface ,\n";
-        manifestString += "                      <" LV2_PROGRAMS__Interface "> ;\n";
-#  else
-        manifestString += "                      ui:showInterface ;\n";
-#  endif
-        manifestString += "\n";
-#  if DISTRHO_PLUGIN_HAS_EMBED_UI
-        manifestString += "    lv2:optionalFeature ui:noUserResize ,\n";
-        manifestString += "                        ui:resize ,\n";
-        manifestString += "                        ui:touch ;\n";
-        manifestString += "\n";
-#  endif
-        manifestString += "    lv2:requiredFeature <" LV2_DATA_ACCESS_URI "> ,\n";
-        manifestString += "                        <" LV2_INSTANCE_ACCESS_URI "> ,\n";
-        manifestString += "                        <" LV2_OPTIONS__options "> ,\n";
-        manifestString += "                        <" LV2_URID__map "> .\n";
-# else
+        addAttribute(manifestString, "lv2:extensionData", lv2ManifestUiExtensionData, 4);
+        addAttribute(manifestString, "lv2:optionalFeature", lv2ManifestUiOptionalFeatures, 4);
+        addAttribute(manifestString, "lv2:requiredFeature", lv2ManifestUiRequiredFeatures, 4);
+        addAttribute(manifestString, "opts:supportedOption", lv2ManifestUiSupportedOptions, 4, true);
+# else // DISTRHO_PLUGIN_WANT_DIRECT_ACCESS
         manifestString += "    rdfs:seeAlso <" + uiTTL + "> .\n";
-# endif
+# endif // DISTRHO_PLUGIN_WANT_DIRECT_ACCESS
         manifestString += "\n";
 #endif
 
@@ -192,7 +316,7 @@ void lv2_generate_ttl(const char* const basename)
         String pluginString;
 
         // header
-#if DISTRHO_LV2_USE_EVENTS_IN
+#if DISTRHO_LV2_USE_EVENTS_IN || DISTRHO_LV2_USE_EVENTS_OUT
         pluginString += "@prefix atom: <" LV2_ATOM_PREFIX "> .\n";
 #endif
         pluginString += "@prefix doap: <http://usefulinc.com/ns/doap#> .\n";
@@ -201,8 +325,12 @@ void lv2_generate_ttl(const char* const basename)
 #ifdef DISTRHO_PLUGIN_BRAND
         pluginString += "@prefix mod:  <http://moddevices.com/ns/mod#> .\n";
 #endif
+        pluginString += "@prefix opts: <" LV2_OPTIONS_PREFIX "> .\n";
+        pluginString += "@prefix rdf:  <http://www.w3.org/1999/02/22-rdf-syntax-ns#> .\n";
         pluginString += "@prefix rdfs: <http://www.w3.org/2000/01/rdf-schema#> .\n";
+#if DISTRHO_LV2_USE_EVENTS_IN || DISTRHO_LV2_USE_EVENTS_OUT
         pluginString += "@prefix rsz:  <" LV2_RESIZE_PORT_PREFIX "> .\n";
+#endif
 #if DISTRHO_PLUGIN_HAS_UI
         pluginString += "@prefix ui:   <" LV2_UI_PREFIX "> .\n";
 #endif
@@ -220,39 +348,10 @@ void lv2_generate_ttl(const char* const basename)
 #endif
         pluginString += "\n";
 
-        // extensionData
-        pluginString += "    lv2:extensionData <" LV2_STATE__interface "> ";
-#if DISTRHO_PLUGIN_WANT_STATE
-        pluginString += ",\n                      <" LV2_OPTIONS__interface "> ";
-        pluginString += ",\n                      <" LV2_WORKER__interface "> ";
-#endif
-#if DISTRHO_PLUGIN_WANT_PROGRAMS
-        pluginString += ",\n                      <" LV2_PROGRAMS__Interface "> ";
-#endif
-#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
-        pluginString += ",\n                      <" MOD_LICENSE__interface "> ";
-#endif
-        pluginString += ";\n\n";
-
-        // optionalFeatures
-#if DISTRHO_PLUGIN_IS_RT_SAFE
-        pluginString += "    lv2:optionalFeature <" LV2_CORE__hardRTCapable "> ,\n";
-        pluginString += "                        <" LV2_BUF_SIZE__boundedBlockLength "> ;\n";
-#else
-        pluginString += "    lv2:optionalFeature <" LV2_BUF_SIZE__boundedBlockLength "> ;\n";
-#endif
-        pluginString += "\n";
-
-        // requiredFeatures
-        pluginString += "    lv2:requiredFeature <" LV2_OPTIONS__options "> ";
-        pluginString += ",\n                        <" LV2_URID__map "> ";
-#if DISTRHO_PLUGIN_WANT_STATE
-        pluginString += ",\n                        <" LV2_WORKER__schedule "> ";
-#endif
-#ifdef DISTRHO_PLUGIN_LICENSED_FOR_MOD
-        pluginString += ",\n                        <" MOD_LICENSE__feature "> ";
-#endif
-        pluginString += ";\n\n";
+        addAttribute(pluginString, "lv2:extensionData", lv2ManifestPluginExtensionData, 4);
+        addAttribute(pluginString, "lv2:optionalFeature", lv2ManifestPluginOptionalFeatures, 4);
+        addAttribute(pluginString, "lv2:requiredFeature", lv2ManifestPluginRequiredFeatures, 4);
+        addAttribute(pluginString, "opts:supportedOption", lv2ManifestPluginSupportedOptions, 4);
 
         // UI
 #if DISTRHO_PLUGIN_HAS_UI
@@ -391,7 +490,7 @@ void lv2_generate_ttl(const char* const basename)
                 bool designated = false;
 
                 // designation
-                if (! plugin.isParameterOutput(i))
+                if (plugin.isParameterInput(i))
                 {
                     switch (plugin.getParameterDesignation(i))
                     {
@@ -410,10 +509,10 @@ void lv2_generate_ttl(const char* const basename)
                     }
                 }
 
-                // name and symbol
                 if (! designated)
                 {
-                    pluginString += "        lv2:name \"" + plugin.getParameterName(i) + "\" ;\n";
+                    // name and symbol
+                    pluginString += "        lv2:name \"\"\"" + plugin.getParameterName(i) + "\"\"\" ;\n";
 
                     String symbol(plugin.getParameterSymbol(i));
 
@@ -421,32 +520,66 @@ void lv2_generate_ttl(const char* const basename)
                         symbol = "lv2_port_" + String(portIndex-1);
 
                     pluginString += "        lv2:symbol \"" + symbol + "\" ;\n";
-                }
 
-                // ranges
-                if (! designated)
-                {
+                    // short name
+                    const String& shortName(plugin.getParameterShortName(i));
+
+                    if (shortName.isNotEmpty())
+                        pluginString += "        lv2:shortName \"\"\"" + shortName + "\"\"\" ;\n";
+
+                    // ranges
                     const ParameterRanges& ranges(plugin.getParameterRanges(i));
 
                     if (plugin.getParameterHints(i) & kParameterIsInteger)
                     {
-                        if (! plugin.isParameterOutput(i))
+                        if (plugin.isParameterInput(i))
                             pluginString += "        lv2:default " + String(int(plugin.getParameterValue(i))) + " ;\n";
                         pluginString += "        lv2:minimum " + String(int(ranges.min)) + " ;\n";
                         pluginString += "        lv2:maximum " + String(int(ranges.max)) + " ;\n";
                     }
                     else
                     {
-                        if (! plugin.isParameterOutput(i))
+                        if (plugin.isParameterInput(i))
                             pluginString += "        lv2:default " + String(plugin.getParameterValue(i)) + " ;\n";
                         pluginString += "        lv2:minimum " + String(ranges.min) + " ;\n";
                         pluginString += "        lv2:maximum " + String(ranges.max) + " ;\n";
                     }
-                }
 
-                // unit
-                if (! designated)
-                {
+                    // enumeration
+                    const ParameterEnumerationValues& enumValues(plugin.getParameterEnumValues(i));
+
+                    if (enumValues.count > 0)
+                    {
+                        if (enumValues.count >= 2 && enumValues.restrictedMode)
+                            pluginString += "        lv2:portProperty lv2:enumeration ;\n";
+
+                        for (uint8_t j=0; j < enumValues.count; ++j)
+                        {
+                            const ParameterEnumerationValue& enumValue(enumValues.values[j]);
+
+                            if (j == 0)
+                                pluginString += "        lv2:scalePoint [\n";
+                            else
+                                pluginString += "        [\n";
+
+                            pluginString += "            rdfs:label  \"\"\"" + enumValue.label + "\"\"\" ;\n";
+
+                            if (plugin.getParameterHints(i) & kParameterIsInteger) {
+                                const int roundedValue = (int)(enumValue.value + 0.5f);
+                                pluginString += "            rdf:value " + String(roundedValue) + " ;\n";
+                            }
+                            else {
+                                pluginString += "            rdf:value " + String(enumValue.value) + " ;\n";
+                            }
+
+                            if (j+1 == enumValues.count)
+                                pluginString += "        ] ;\n\n";
+                            else
+                                pluginString += "        ] ,\n";
+                        }
+                    }
+
+                    // unit
                     const String& unit(plugin.getParameterUnit(i));
 
                     if (! unit.isEmpty())
@@ -482,31 +615,39 @@ void lv2_generate_ttl(const char* const basename)
                         else
                         {
                             pluginString += "        unit:unit [\n";
+                            pluginString += "            a unit:Unit ;\n";
                             pluginString += "            rdfs:label  \"" + unit + "\" ;\n";
                             pluginString += "            unit:symbol \"" + unit + "\" ;\n";
                             pluginString += "            unit:render \"%f " + unit + "\" ;\n";
                             pluginString += "        ] ;\n";
                         }
                     }
-                }
 
-                // hints
-                if (! designated)
-                {
+                    // comment
+                    const String& comment(plugin.getParameterDescription(i));
+
+                    if (comment.isNotEmpty())
+                        pluginString += "        rdfs:comment \"\"\"" + comment + "\"\"\" ;\n";
+
+                    // hints
                     const uint32_t hints(plugin.getParameterHints(i));
 
                     if (hints & kParameterIsBoolean)
+                    {
+                        if ((hints & kParameterIsTrigger) == kParameterIsTrigger)
+                            pluginString += "        lv2:portProperty <" LV2_PORT_PROPS__trigger "> ;\n";
                         pluginString += "        lv2:portProperty lv2:toggled ;\n";
+                    }
                     if (hints & kParameterIsInteger)
                         pluginString += "        lv2:portProperty lv2:integer ;\n";
                     if (hints & kParameterIsLogarithmic)
                         pluginString += "        lv2:portProperty <" LV2_PORT_PROPS__logarithmic "> ;\n";
-                    if ((hints & kParameterIsAutomable) == 0 && ! plugin.isParameterOutput(i))
+                    if ((hints & kParameterIsAutomable) == 0 && plugin.isParameterInput(i))
                     {
                         pluginString += "        lv2:portProperty <" LV2_PORT_PROPS__expensive "> ,\n";
                         pluginString += "                         <" LV2_KXSTUDIO_PROPERTIES__NonAutomable "> ;\n";
                     }
-                }
+                } // ! designated
 
                 if (i+1 == count)
                     pluginString += "    ] ;\n\n";
@@ -520,7 +661,7 @@ void lv2_generate_ttl(const char* const basename)
             const String comment(plugin.getDescription());
 
             if (comment.isNotEmpty())
-                pluginString += "    rdfs:comment \"\"\"\n" + comment + "\n\"\"\" ;\n\n";
+                pluginString += "    rdfs:comment \"\"\"" + comment + "\"\"\" ;\n\n";
         }
 
 #ifdef DISTRHO_PLUGIN_BRAND
@@ -530,7 +671,7 @@ void lv2_generate_ttl(const char* const basename)
 #endif
 
         // name
-        pluginString += "    doap:name \"" + String(plugin.getName()) + "\" ;\n";
+        pluginString += "    doap:name \"\"\"" + String(plugin.getName()) + "\"\"\" ;\n";
 
         // license
         {
@@ -539,7 +680,7 @@ void lv2_generate_ttl(const char* const basename)
             if (license.contains("://"))
                 pluginString += "    doap:license <" +  license + "> ;\n\n";
             else
-                pluginString += "    doap:license \"" +  license + "\" ;\n\n";
+                pluginString += "    doap:license \"\"\"" +  license + "\"\"\" ;\n\n";
         }
 
         // developer
@@ -547,7 +688,7 @@ void lv2_generate_ttl(const char* const basename)
             const String homepage(plugin.getHomePage());
 
             pluginString += "    doap:maintainer [\n";
-            pluginString += "        foaf:name \"" + String(plugin.getMaker()) + "\" ;\n";
+            pluginString += "        foaf:name \"\"\"" + String(plugin.getMaker()) + "\"\"\" ;\n";
 
             if (homepage.isNotEmpty())
                 pluginString += "        foaf:homepage <" + homepage + "> ;\n";
@@ -583,27 +724,17 @@ void lv2_generate_ttl(const char* const basename)
         std::fstream uiFile(uiTTL, std::ios::out);
 
         String uiString;
-        uiString += "@prefix lv2: <" LV2_CORE_PREFIX "> .\n";
-        uiString += "@prefix ui:  <" LV2_UI_PREFIX "> .\n";
+        uiString += "@prefix lv2:  <" LV2_CORE_PREFIX "> .\n";
+        uiString += "@prefix ui:   <" LV2_UI_PREFIX "> .\n";
+        uiString += "@prefix opts: <" LV2_OPTIONS_PREFIX "> .\n";
         uiString += "\n";
 
         uiString += "<" DISTRHO_UI_URI ">\n";
-        uiString += "    lv2:extensionData ui:idleInterface ,\n";
-#  if DISTRHO_PLUGIN_WANT_PROGRAMS
-        uiString += "                      ui:showInterface ,\n";
-        uiString += "                      <" LV2_PROGRAMS__Interface "> ;\n";
-#  else
-        uiString += "                      ui:showInterface ;\n";
-#  endif
-        uiString += "\n";
-#  if DISTRHO_PLUGIN_HAS_EMBED_UI
-        uiString += "    lv2:optionalFeature ui:noUserResize ,\n";
-        uiString += "                        ui:resize ,\n";
-        uiString += "                        ui:touch ;\n";
-        uiString += "\n";
-#  endif
-        uiString += "    lv2:requiredFeature <" LV2_OPTIONS__options "> ,\n";
-        uiString += "                        <" LV2_URID__map "> .\n";
+
+        addAttribute(uiString, "lv2:extensionData", lv2ManifestUiExtensionData, 4);
+        addAttribute(uiString, "lv2:optionalFeature", lv2ManifestUiOptionalFeatures, 4);
+        addAttribute(uiString, "lv2:requiredFeature", lv2ManifestUiRequiredFeatures, 4);
+        addAttribute(uiString, "opts:supportedOption", lv2ManifestUiSupportedOptions, 4, true);
 
         uiFile << uiString << std::endl;
         uiFile.close();
@@ -670,7 +801,7 @@ void lv2_generate_ttl(const char* const basename)
                 if (value.length() < 10)
                     presetString += " \"" + value + "\" ;\n";
                 else
-                    presetString += "\n\"\"\"\n" + value + "\n\"\"\" ;\n";
+                    presetString += "\n\"\"\"" + value + "\"\"\" ;\n";
             }
 
             if (numParameters > 0)
@@ -703,7 +834,7 @@ void lv2_generate_ttl(const char* const basename)
                 else
                     presetString += "        pset:value " + String(plugin.getParameterValue(j)) + " ;\n";
 
-                if (j+1 == numParameters || (j+2 == numParameters && plugin.isParameterOutput(j+1)))
+                if (j+1 == numParameters || plugin.isParameterOutput(j+1))
                     presetString += "    ] .\n\n";
                 else
                     presetString += "    ] ,\n";
