@@ -1,6 +1,6 @@
 /*
  * DISTRHO Plugin Framework (DPF)
- * Copyright (C) 2012-2016 Filipe Coelho <falktx@falktx.com>
+ * Copyright (C) 2012-2018 Filipe Coelho <falktx@falktx.com>
  *
  * Permission to use, copy, modify, and/or distribute this software for any purpose with
  * or without fee is hereby granted, provided that the above copyright notice and this
@@ -29,19 +29,15 @@
 # define __cdecl
 #endif
 
-// has some conflicts
-#ifdef noexcept
-# undef noexcept
-#endif
-
 #define VESTIGE_HEADER
 #define VST_FORCE_DEPRECATED 0
 
+#include <clocale>
 #include <map>
 #include <string>
 
 #ifdef VESTIGE_HEADER
-# include "vestige/aeffectx.h"
+# include "vestige/vestige.h"
 #define effFlagsProgramChunks (1 << 5)
 #define effSetProgramName 4
 #define effGetParamLabel 6
@@ -51,11 +47,8 @@
 #define effCanBeAutomated 26
 #define effGetProgramNameIndexed 29
 #define effGetPlugCategory 35
-#define effIdle 53
 #define effEditKeyDown 59
 #define effEditKeyUp 60
-#define kPlugCategEffect 1
-#define kPlugCategSynth 2
 #define kVstVersion 2400
 struct ERect {
     int16_t top, left, bottom, right;
@@ -68,31 +61,79 @@ START_NAMESPACE_DISTRHO
 
 typedef std::map<const String, String> StringMap;
 
+static const int kVstMidiEventSize = static_cast<int>(sizeof(VstMidiEvent));
+
+#if ! DISTRHO_PLUGIN_WANT_MIDI_OUTPUT
+static const writeMidiFunc writeMidiCallback = nullptr;
+#endif
+
 // -----------------------------------------------------------------------
 
 void strncpy(char* const dst, const char* const src, const size_t size)
 {
-    std::strncpy(dst, src, size-1);
-    dst[size-1] = '\0';
+    DISTRHO_SAFE_ASSERT_RETURN(size > 0,);
+
+    if (const size_t len = std::min(std::strlen(src), size-1U))
+    {
+        std::memcpy(dst, src, len);
+        dst[len] = '\0';
+    }
+    else
+    {
+        dst[0] = '\0';
+    }
 }
 
 void snprintf_param(char* const dst, const float value, const size_t size)
 {
+    DISTRHO_SAFE_ASSERT_RETURN(size > 0,);
     std::snprintf(dst, size-1, "%f", value);
     dst[size-1] = '\0';
 }
 
-#if DISTRHO_PLUGIN_HAS_UI
+void snprintf_iparam(char* const dst, const int32_t value, const size_t size)
+{
+    DISTRHO_SAFE_ASSERT_RETURN(size > 0,);
+    std::snprintf(dst, size-1, "%d", value);
+    dst[size-1] = '\0';
+}
+
 // -----------------------------------------------------------------------
 
-class UiHelper
+class ScopedSafeLocale {
+public:
+    ScopedSafeLocale() noexcept
+        : locale(::strdup(::setlocale(LC_NUMERIC, nullptr)))
+    {
+        ::setlocale(LC_NUMERIC, "C");
+    }
+
+    ~ScopedSafeLocale() noexcept
+    {
+        if (locale != nullptr)
+        {
+            ::setlocale(LC_NUMERIC, locale);
+            std::free(locale);
+        }
+    }
+
+private:
+    char* const locale;
+
+    DISTRHO_DECLARE_NON_COPY_CLASS(ScopedSafeLocale)
+    DISTRHO_PREVENT_HEAP_ALLOCATION
+};
+
+// -----------------------------------------------------------------------
+
+class ParameterCheckHelper
 {
 public:
-    UiHelper()
+    ParameterCheckHelper()
         : parameterChecks(nullptr),
           parameterValues(nullptr) {}
 
-    virtual ~UiHelper()
+    virtual ~ParameterCheckHelper()
     {
         if (parameterChecks != nullptr)
         {
@@ -106,25 +147,26 @@ public:
         }
     }
 
-    bool*   parameterChecks;
-    float*  parameterValues;
+    bool*  parameterChecks;
+    float* parameterValues;
 
-# if DISTRHO_PLUGIN_WANT_STATE
+#if DISTRHO_PLUGIN_WANT_STATE
     virtual void setStateFromUI(const char* const newKey, const char* const newValue) = 0;
-# endif
+#endif
 };
 
+#if DISTRHO_PLUGIN_HAS_UI
 // -----------------------------------------------------------------------
 
 class UIVst
 {
 public:
-    UIVst(const audioMasterCallback audioMaster, AEffect* const effect, UiHelper* const uiHelper, PluginExporter* const plugin, const intptr_t winId)
+    UIVst(const audioMasterCallback audioMaster, AEffect* const effect, ParameterCheckHelper* const uiHelper, PluginExporter* const plugin, const intptr_t winId, const float scaleFactor)
         : fAudioMaster(audioMaster),
           fEffect(effect),
           fUiHelper(uiHelper),
           fPlugin(plugin),
-          fUI(this, winId, editParameterCallback, setParameterCallback, setStateCallback, sendNoteCallback, setSizeCallback, plugin->getInstancePointer()),
+          fUI(this, winId, editParameterCallback, setParameterCallback, setStateCallback, sendNoteCallback, setSizeCallback, scaleFactor, plugin->getInstancePointer()),
           fShouldCaptureVstKeys(false)
     {
         // FIXME only needed for windows?
@@ -189,6 +231,8 @@ public:
 
         d_stdout("handlePluginKeyEvent %i %i %li\n", down, index, (long int)value);
 
+        using namespace DGL_NAMESPACE;
+
         int special = 0;
         switch (value)
         {
@@ -294,7 +338,7 @@ private:
     // Vst stuff
     const audioMasterCallback fAudioMaster;
     AEffect* const fEffect;
-    UiHelper* const fUiHelper;
+    ParameterCheckHelper* const fUiHelper;
     PluginExporter* const fPlugin;
 
     // Plugin UI
@@ -337,15 +381,12 @@ private:
 
 // -----------------------------------------------------------------------
 
-#if DISTRHO_PLUGIN_HAS_UI
-class PluginVst : public UiHelper
-#else
-class PluginVst
-#endif
+class PluginVst : public ParameterCheckHelper
 {
 public:
     PluginVst(const audioMasterCallback audioMaster, AEffect* const effect)
-        : fAudioMaster(audioMaster),
+        : fPlugin(this, writeMidiCallback),
+          fAudioMaster(audioMaster),
           fEffect(effect)
     {
         std::memset(fProgramName, 0, sizeof(char)*(32+1));
@@ -370,14 +411,16 @@ public:
             for (uint32_t i=0; i < paramCount; ++i)
             {
                 parameterChecks[i] = false;
-                parameterValues[i] = 0.0f;
+                parameterValues[i] = NAN;
             }
         }
 # if DISTRHO_OS_MAC
 #  ifdef __LP64__
         fUsingNsView = true;
 #  else
-#  warning 32bit VST UIs on OSX only work if the host supports "hasCockosViewAsConfig"
+#   ifndef DISTRHO_NO_WARNINGS
+#    warning 32bit VST UIs on OSX only work if the host supports "hasCockosViewAsConfig"
+#   endif
         fUsingNsView = false;
 #  endif
 # endif // DISTRHO_OS_MAC
@@ -421,7 +464,7 @@ public:
         case effSetProgramName:
             if (char* const programName = (char*)ptr)
             {
-                DISTRHO::strncpy(fProgramName, programName, 32);
+                DISTRHO_NAMESPACE::strncpy(fProgramName, programName, 32);
                 return 1;
             }
             break;
@@ -429,7 +472,7 @@ public:
         case effGetProgramName:
             if (char* const programName = (char*)ptr)
             {
-                DISTRHO::strncpy(programName, fProgramName, 24);
+                DISTRHO_NAMESPACE::strncpy(programName, fProgramName, 24);
                 return 1;
             }
             break;
@@ -437,7 +480,7 @@ public:
         case effGetProgramNameIndexed:
             if (char* const programName = (char*)ptr)
             {
-                DISTRHO::strncpy(programName, fProgramName, 24);
+                DISTRHO_NAMESPACE::strncpy(programName, fProgramName, 24);
                 return 1;
             }
             break;
@@ -445,7 +488,37 @@ public:
         case effGetParamDisplay:
             if (ptr != nullptr && index < static_cast<int32_t>(fPlugin.getParameterCount()))
             {
-                DISTRHO::snprintf_param((char*)ptr, fPlugin.getParameterValue(index), 24);
+                const uint32_t hints = fPlugin.getParameterHints(index);
+                float value = fPlugin.getParameterValue(index);
+
+                if (hints & kParameterIsBoolean)
+                {
+                    const ParameterRanges& ranges(fPlugin.getParameterRanges(index));
+                    const float midRange = ranges.min + (ranges.max - ranges.min) / 2.0f;
+
+                    value = value > midRange ? ranges.max : ranges.min;
+                }
+                else if (hints & kParameterIsInteger)
+                {
+                    value = std::round(value);
+                }
+
+                const ParameterEnumerationValues& enumValues(fPlugin.getParameterEnumValues(index));
+
+                for (uint8_t i = 0; i < enumValues.count; ++i)
+                {
+                    if (d_isNotEqual(value, enumValues.values[i].value))
+                        continue;
+
+                    DISTRHO_NAMESPACE::strncpy((char*)ptr, enumValues.values[i].label.buffer(), 24);
+                    return 1;
+                }
+
+                if (hints & kParameterIsInteger)
+                    DISTRHO_NAMESPACE::snprintf_iparam((char*)ptr, (int32_t)value, 24);
+                else
+                    DISTRHO_NAMESPACE::snprintf_param((char*)ptr, value, 24);
+
                 return 1;
             }
             break;
@@ -505,7 +578,10 @@ public:
             {
                 d_lastUiSampleRate = fPlugin.getSampleRate();
 
-                UIExporter tmpUI(nullptr, 0, nullptr, nullptr, nullptr, nullptr, nullptr, fPlugin.getInstancePointer());
+                // TODO
+                const float scaleFactor = 1.0f;
+
+                UIExporter tmpUI(nullptr, 0, nullptr, nullptr, nullptr, nullptr, nullptr, scaleFactor, fPlugin.getInstancePointer());
                 fVstRect.right  = tmpUI.getWidth();
                 fVstRect.bottom = tmpUI.getHeight();
                 tmpUI.quit();
@@ -514,7 +590,8 @@ public:
             return 1;
 
         case effEditOpen:
-            if (fVstUI == nullptr)
+            delete fVstUI; // hosts which don't pair effEditOpen/effEditClose calls (Minihost Modular)
+            fVstUI = nullptr;
             {
 # if DISTRHO_OS_MAC
                 if (! fUsingNsView)
@@ -525,7 +602,10 @@ public:
 # endif
                 d_lastUiSampleRate = fPlugin.getSampleRate();
 
-                fVstUI = new UIVst(fAudioMaster, fEffect, this, &fPlugin, (intptr_t)ptr);
+                // TODO
+                const float scaleFactor = 1.0f;
+
+                fVstUI = new UIVst(fAudioMaster, fEffect, this, &fPlugin, (intptr_t)ptr, scaleFactor);
 
 # if DISTRHO_PLUGIN_WANT_FULL_STATE
                 // Update current state from plugin side
@@ -582,6 +662,7 @@ public:
 
 #if DISTRHO_PLUGIN_WANT_STATE
         case effGetChunk:
+        {
             if (ptr == nullptr)
                 return 0;
 
@@ -591,7 +672,9 @@ public:
                 fStateChunk = nullptr;
             }
 
-            if (fPlugin.getStateCount() == 0)
+            const uint32_t paramCount = fPlugin.getParameterCount();
+
+            if (fPlugin.getStateCount() == 0 && paramCount == 0)
             {
                 fStateChunk    = new char[1];
                 fStateChunk[0] = '\0';
@@ -625,6 +708,30 @@ public:
                     chunkStr += tmpStr;
                 }
 
+                if (paramCount != 0)
+                {
+                    // add another separator
+                    chunkStr += "\xff";
+
+                    // temporarily set locale to "C" while converting floats
+                    const ScopedSafeLocale ssl;
+
+                    for (uint32_t i=0; i<paramCount; ++i)
+                    {
+                        if (fPlugin.isParameterOutputOrTrigger(i))
+                            continue;
+
+                        // join key and value
+                        String tmpStr;
+                        tmpStr  = fPlugin.getParameterSymbol(i);
+                        tmpStr += "\xff";
+                        tmpStr += String(fPlugin.getParameterValue(i));
+                        tmpStr += "\xff";
+
+                        chunkStr += tmpStr;
+                    }
+                }
+
                 const std::size_t chunkSize(chunkStr.length()+1);
 
                 fStateChunk = new char[chunkSize];
@@ -642,21 +749,27 @@ public:
 
             *(void**)ptr = fStateChunk;
             return ret;
+        }
 
         case effSetChunk:
         {
             if (value <= 1 || ptr == nullptr)
                 return 0;
 
+            const size_t chunkSize = static_cast<size_t>(value);
+
             const char* key   = (const char*)ptr;
             const char* value = nullptr;
+            size_t size, bytesRead = 0;
 
-            for (;;)
+            while (bytesRead < chunkSize)
             {
                 if (key[0] == '\0')
                     break;
 
-                value = key+(std::strlen(key)+1);
+                size  = std::strlen(key)+1;
+                value = key + size;
+                bytesRead += size;
 
                 setStateFromUI(key, value);
 
@@ -666,7 +779,52 @@ public:
 # endif
 
                 // get next key
-                key = value+(std::strlen(value)+1);
+                size = std::strlen(value)+1;
+                key  = value + size;
+                bytesRead += size;
+            }
+
+            const uint32_t paramCount = fPlugin.getParameterCount();
+
+            if (bytesRead+4 < chunkSize && paramCount != 0)
+            {
+                ++key;
+                float fvalue;
+
+                // temporarily set locale to "C" while converting floats
+                const ScopedSafeLocale ssl;
+
+                while (bytesRead < chunkSize)
+                {
+                    if (key[0] == '\0')
+                        break;
+
+                    size  = std::strlen(key)+1;
+                    value = key + size;
+                    bytesRead += size;
+
+                    // find parameter with this symbol, and set its value
+                    for (uint32_t i=0; i<paramCount; ++i)
+                    {
+                        if (fPlugin.isParameterOutputOrTrigger(i))
+                            continue;
+                        if (fPlugin.getParameterSymbol(i) != key)
+                            continue;
+
+                        fvalue = std::atof(value);
+                        fPlugin.setParameterValue(i, fvalue);
+# if DISTRHO_PLUGIN_HAS_UI
+                        if (fVstUI != nullptr)
+                            setParameterValueFromPlugin(i, fvalue);
+# endif
+                        break;
+                    }
+
+                    // get next key
+                    size = std::strlen(value)+1;
+                    key  = value + size;
+                    bytesRead += size;
+                }
             }
 
             return 1;
@@ -767,8 +925,23 @@ public:
 
     void vst_setParameter(const int32_t index, const float value)
     {
+        const uint32_t hints(fPlugin.getParameterHints(index));
         const ParameterRanges& ranges(fPlugin.getParameterRanges(index));
-        const float realValue(ranges.getUnnormalizedValue(value));
+
+        // TODO figure out how to detect kVstParameterUsesIntegerMinMax host support, and skip normalization
+        float realValue = ranges.getUnnormalizedValue(value);
+
+        if (hints & kParameterIsBoolean)
+        {
+            const float midRange = ranges.min + (ranges.max - ranges.min) / 2.0f;
+            realValue = realValue > midRange ? ranges.max : ranges.min;
+        }
+
+        if (hints & kParameterIsInteger)
+        {
+            realValue = std::round(realValue);
+        }
+
         fPlugin.setParameterValue(index, realValue);
 
 #if DISTRHO_PLUGIN_HAS_UI
@@ -780,7 +953,10 @@ public:
     void vst_processReplacing(const float** const inputs, float** const outputs, const int32_t sampleFrames)
     {
         if (sampleFrames <= 0)
+        {
+            updateParameterOutputsAndTriggers();
             return;
+        }
 
         if (! fPlugin.isActive())
         {
@@ -807,15 +983,23 @@ public:
 
             if (vstTimeInfo->flags & (kVstPpqPosValid|kVstTimeSigValid))
             {
+                const double ppqPos    = std::abs(vstTimeInfo->ppqPos);
                 const int    ppqPerBar = vstTimeInfo->timeSigNumerator * 4 / vstTimeInfo->timeSigDenominator;
-                const double barBeats  = (std::fmod(vstTimeInfo->ppqPos, ppqPerBar) / ppqPerBar) * vstTimeInfo->timeSigDenominator;
+                const double barBeats  = (std::fmod(ppqPos, ppqPerBar) / ppqPerBar) * vstTimeInfo->timeSigNumerator;
                 const double rest      =  std::fmod(barBeats, 1.0);
 
-                fTimePosition.bbt.bar         = int(vstTimeInfo->ppqPos)/ppqPerBar + 1;
-                fTimePosition.bbt.beat        = barBeats-rest+1;
-                fTimePosition.bbt.tick        = rest*fTimePosition.bbt.ticksPerBeat+0.5;
+                fTimePosition.bbt.bar         = static_cast<int32_t>(ppqPos) / ppqPerBar + 1;
+                fTimePosition.bbt.beat        = static_cast<int32_t>(barBeats - rest + 0.5) + 1;
+                fTimePosition.bbt.tick        = static_cast<int32_t>(rest * fTimePosition.bbt.ticksPerBeat + 0.5);
                 fTimePosition.bbt.beatsPerBar = vstTimeInfo->timeSigNumerator;
                 fTimePosition.bbt.beatType    = vstTimeInfo->timeSigDenominator;
+
+                if (vstTimeInfo->ppqPos < 0.0)
+                {
+                    --fTimePosition.bbt.bar;
+                    fTimePosition.bbt.beat = vstTimeInfo->timeSigNumerator - fTimePosition.bbt.beat + 1;
+                    fTimePosition.bbt.tick = int(fTimePosition.bbt.ticksPerBeat) - fTimePosition.bbt.tick - 1;
+                }
             }
             else
             {
@@ -839,16 +1023,7 @@ public:
         fPlugin.run(inputs, outputs, sampleFrames);
 #endif
 
-#if DISTRHO_PLUGIN_HAS_UI
-        if (fVstUI == nullptr)
-            return;
-
-        for (uint32_t i=0, count=fPlugin.getParameterCount(); i < count; ++i)
-        {
-            if (fPlugin.isParameterOutput(i))
-                setParameterValueFromPlugin(i, fPlugin.getParameterValue(i));
-        }
-#endif
+        updateParameterOutputsAndTriggers();
     }
 
     // -------------------------------------------------------------------
@@ -856,13 +1031,13 @@ public:
     friend class UIVst;
 
 private:
+    // Plugin
+    PluginExporter fPlugin;
+
     // VST stuff
     const audioMasterCallback fAudioMaster;
     AEffect* const fEffect;
 
-    // Plugin
-    PluginExporter fPlugin;
-
     // Temporary data
     char fProgramName[32+1];
 
@@ -904,6 +1079,56 @@ private:
     // -------------------------------------------------------------------
     // functions called from the plugin side, RT no block
 
+    void updateParameterOutputsAndTriggers()
+    {
+        float curValue;
+
+        for (uint32_t i=0, count=fPlugin.getParameterCount(); i < count; ++i)
+        {
+            if (fPlugin.isParameterOutput(i))
+            {
+                // NOTE: no output parameter support in VST, simulate it here
+                curValue = fPlugin.getParameterValue(i);
+
+                if (d_isEqual(curValue, parameterValues[i]))
+                    continue;
+
+#if DISTRHO_PLUGIN_HAS_UI
+                if (fVstUI != nullptr)
+                    setParameterValueFromPlugin(i, curValue);
+                else
+#endif
+                parameterValues[i] = curValue;
+
+#ifndef DPF_VST_SHOW_PARAMETER_OUTPUTS
+                // skip automating parameter outputs from plugin if we disable them on VST
+                continue;
+#endif
+            }
+            else if ((fPlugin.getParameterHints(i) & kParameterIsTrigger) == kParameterIsTrigger)
+            {
+                // NOTE: no trigger support in VST parameters, simulate it here
+                curValue = fPlugin.getParameterValue(i);
+
+                if (d_isEqual(curValue, fPlugin.getParameterRanges(i).def))
+                    continue;
+
+#if DISTRHO_PLUGIN_HAS_UI
+                if (fVstUI != nullptr)
+                    setParameterValueFromPlugin(i, curValue);
+#endif
+                fPlugin.setParameterValue(i, curValue);
+            }
+            else
+            {
+                continue;
+            }
+
+            const ParameterRanges& ranges(fPlugin.getParameterRanges(i));
+            hostCallback(audioMasterAutomate, i, 0, nullptr, ranges.getNormalizedValue(curValue));
+        }
+    }
+
 #if DISTRHO_PLUGIN_HAS_UI
     void setParameterValueFromPlugin(const uint32_t index, const float realValue)
     {
@@ -912,6 +1137,37 @@ private:
     }
 #endif
 
+#if DISTRHO_PLUGIN_WANT_MIDI_OUTPUT
+    bool writeMidi(const MidiEvent& midiEvent)
+    {
+        if (midiEvent.size > 4)
+            return true;
+
+        VstEvents vstEvents;
+        std::memset(&vstEvents, 0, sizeof(VstEvents));
+
+        VstMidiEvent vstMidiEvent;
+        std::memset(&vstMidiEvent, 0, sizeof(VstMidiEvent));
+
+        vstEvents.numEvents = 1;
+        vstEvents.events[0] = (VstEvent*)&vstMidiEvent;
+
+        vstMidiEvent.type = kVstMidiType;
+        vstMidiEvent.byteSize    = kVstMidiEventSize;
+        vstMidiEvent.deltaFrames = midiEvent.frame;
+
+        for (uint8_t i=0; i<midiEvent.size; ++i)
+            vstMidiEvent.midiData[i] = midiEvent.data[i];
+
+        return hostCallback(audioMasterProcessEvents, 0, 0, &vstEvents) == 1;
+    }
+
+    static bool writeMidiCallback(void* ptr, const MidiEvent& midiEvent)
+    {
+        return ((PluginVst*)ptr)->writeMidi(midiEvent);
+    }
+#endif
+
 #if DISTRHO_PLUGIN_WANT_STATE
     // -------------------------------------------------------------------
     // functions called from the UI side, may block
@@ -952,17 +1208,10 @@ struct VstObject {
     PluginVst* plugin;
 };
 
-#ifdef VESTIGE_HEADER
-# define validObject  effect != nullptr && effect->ptr3 != nullptr
-# define validPlugin  effect != nullptr && effect->ptr3 != nullptr && ((VstObject*)effect->ptr3)->plugin != nullptr
-# define vstObjectPtr (VstObject*)effect->ptr3
-#else
-# define validObject  effect != nullptr && effect->object != nullptr
-# define validPlugin  effect != nullptr && effect->object != nullptr && ((VstObject*)effect->object)->plugin != nullptr
-# define vstObjectPtr (VstObject*)effect->object
-#endif
-
-#define pluginPtr     (vstObjectPtr)->plugin
+#define validObject  effect != nullptr && effect->object != nullptr
+#define validPlugin  effect != nullptr && effect->object != nullptr && ((VstObject*)effect->object)->plugin != nullptr
+#define vstObjectPtr (VstObject*)effect->object
+#define pluginPtr    (vstObjectPtr)->plugin
 
 static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t index, intptr_t value, void* ptr, float opt)
 {
@@ -977,7 +1226,7 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     }
 
     // Create dummy plugin to get data from
-    static PluginExporter plugin;
+    static PluginExporter plugin(nullptr, nullptr);
 
     if (doInternalInit)
     {
@@ -1030,11 +1279,7 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
             /* This code invalidates the object created in VSTPluginMain
              * Probably not safe against all hosts */
             obj->audioMaster = nullptr;
-# ifdef VESTIGE_HEADER
-            effect->ptr3 = nullptr;
-# else
-            vstObjectPtr = nullptr;
-# endif
+            effect->object = nullptr;
             delete obj;
 #endif
 
@@ -1046,7 +1291,7 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     case effGetParamLabel:
         if (ptr != nullptr && index < static_cast<int32_t>(plugin.getParameterCount()))
         {
-            DISTRHO::strncpy((char*)ptr, plugin.getParameterUnit(index), 8);
+            DISTRHO_NAMESPACE::strncpy((char*)ptr, plugin.getParameterUnit(index), 8);
             return 1;
         }
         return 0;
@@ -1054,11 +1299,51 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     case effGetParamName:
         if (ptr != nullptr && index < static_cast<int32_t>(plugin.getParameterCount()))
         {
-            DISTRHO::strncpy((char*)ptr, plugin.getParameterName(index), 16);
+            const String& shortName(plugin.getParameterShortName(index));
+            if (shortName.isNotEmpty())
+                DISTRHO_NAMESPACE::strncpy((char*)ptr, shortName, 16);
+            else
+                DISTRHO_NAMESPACE::strncpy((char*)ptr, plugin.getParameterName(index), 16);
             return 1;
         }
         return 0;
 
+    case effGetParameterProperties:
+        if (ptr != nullptr && index < static_cast<int32_t>(plugin.getParameterCount()))
+        {
+            if (VstParameterProperties* const properties = (VstParameterProperties*)ptr)
+            {
+                memset(properties, 0, sizeof(VstParameterProperties));
+
+                const uint32_t hints = plugin.getParameterHints(index);
+
+                if (hints & kParameterIsOutput)
+                    return 1;
+
+                if (hints & kParameterIsBoolean)
+                {
+                    properties->flags |= kVstParameterIsSwitch;
+                }
+
+                if (hints & kParameterIsInteger)
+                {
+                    properties->flags |= kVstParameterUsesIntegerMinMax;
+                    const ParameterRanges& ranges(plugin.getParameterRanges(index));
+
+                    properties->minInteger = static_cast<int32_t>(ranges.min);
+                    properties->maxInteger = static_cast<int32_t>(ranges.max);
+                }
+
+                if (hints & kParameterIsLogarithmic)
+                {
+                    properties->flags |= kVstParameterCanRamp;
+                }
+
+                return 1;
+            }
+        }
+        return 0;
+
     case effGetPlugCategory:
 #if DISTRHO_PLUGIN_IS_SYNTH
         return kPlugCategSynth;
@@ -1069,7 +1354,7 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     case effGetEffectName:
         if (char* const cptr = (char*)ptr)
         {
-            DISTRHO::strncpy(cptr, plugin.getName(), 32);
+            DISTRHO_NAMESPACE::strncpy(cptr, plugin.getName(), 32);
             return 1;
         }
         return 0;
@@ -1077,7 +1362,7 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     case effGetVendorString:
         if (char* const cptr = (char*)ptr)
         {
-            DISTRHO::strncpy(cptr, plugin.getMaker(), 32);
+            DISTRHO_NAMESPACE::strncpy(cptr, plugin.getMaker(), 32);
             return 1;
         }
         return 0;
@@ -1085,7 +1370,7 @@ static intptr_t vst_dispatcherCallback(AEffect* effect, int32_t opcode, int32_t
     case effGetProductString:
         if (char* const cptr = (char*)ptr)
         {
-            DISTRHO::strncpy(cptr, plugin.getLabel(), 32);
+            DISTRHO_NAMESPACE::strncpy(cptr, plugin.getLabel(), 32);
             return 1;
         }
         return 0;
@@ -1165,20 +1450,18 @@ const AEffect* VSTPluginMain(audioMasterCallback audioMaster)
     // vst fields
     effect->magic    = kEffectMagic;
     effect->uniqueID = plugin->getUniqueId();
-#ifdef VESTIGE_HEADER
-    int32_t* const version = (int32_t*)&effect->unknown1;
-    *version = plugin->getVersion();
-#else
-    effect->version = plugin->getVersion();
-#endif
+    effect->version  = plugin->getVersion();
 
-    // VST doesn't support parameter outputs, hide them
+    // VST doesn't support parameter outputs. we can fake them, but it is a hack. Disabled by default.
+#ifdef DPF_VST_SHOW_PARAMETER_OUTPUTS
+    const int numParams = plugin->getParameterCount();
+#else
     int numParams = 0;
     bool outputsReached = false;
 
     for (uint32_t i=0, count=plugin->getParameterCount(); i < count; ++i)
     {
-        if (! plugin->isParameterOutput(i))
+        if (plugin->isParameterInput(i))
         {
             // parameter outputs must be all at the end
             DISTRHO_SAFE_ASSERT_BREAK(! outputsReached);
@@ -1187,6 +1470,7 @@ const AEffect* VSTPluginMain(audioMasterCallback audioMaster)
         }
         outputsReached = true;
     }
+#endif
 
     // plugin fields
     effect->numParams   = numParams;
@@ -1217,11 +1501,9 @@ const AEffect* VSTPluginMain(audioMasterCallback audioMaster)
     VstObject* const obj(new VstObject());
     obj->audioMaster = audioMaster;
     obj->plugin      = nullptr;
-#ifdef VESTIGE_HEADER
-    effect->ptr3   = obj;
-#else
+
+    // done
     effect->object = obj;
-#endif
 
     return effect;
 }
