@@ -5,7 +5,7 @@
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of
- * the License, or any later version.
+ * the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -712,11 +712,6 @@ void ZaMultiCompPlugin::activate()
 		simper[1][i].s[1] = 0.f;
 	}
 	max = 0.f;
-	for (i = 0; i < MAX_SAMPLES; i++) {
-		outlevelold[0][i] = 0.f;
-		outlevelold[1][i] = 0.f;
-		outlevelold[2][i] = 0.f;
-	}
 	pos[0] = 0;
 	pos[1] = 0;
 	pos[2] = 0;
@@ -754,7 +749,7 @@ float ZaMultiCompPlugin::run_linear_svf_xover(struct linear_svf *self, float in,
 	double s1 = self->s[1];
 	double g2 = g*g;
 	double vhigh = in * mixhigh;
-	double vband = in * 0.5;
+	double vband = in * 0.75;
 	double vlow = in * mixlow;
 
 	v[0] = in;
@@ -770,8 +765,8 @@ void ZaMultiCompPlugin::calc_lr4(float f, int i)
 {
 	float srate = getSampleRate();
 
-	linear_svf_set_xover(&simper[0][i], srate, f, 0.5);
-	linear_svf_set_xover(&simper[1][i], srate, f, 0.5);
+	linear_svf_set_xover(&simper[0][i], srate, f, 0.25);
+	linear_svf_set_xover(&simper[1][i], srate, f, 0.25);
 }
 
 void ZaMultiCompPlugin::run_lr4(int i, float in, float *outlo, float *outhi)
@@ -795,10 +790,10 @@ void ZaMultiCompPlugin::run_comp(int k, float in, float *out)
 
         Lyg = 0.f;
 	in = sanitize_denormal(in);
-        Lxg = (in==0.f) ? -160.f : to_dB(fabs(in));
+        Lxg = (in==0.f) ? -160.f : to_dB(fabsf(in));
         Lxg = sanitize_denormal(Lxg);
 
-	checkwidth = 2.f*fabs(Lxg-thresdb[k]);
+	checkwidth = 2.f*fabsf(Lxg-thresdb[k]);
 	if (2.f*(Lxg-thresdb[k]) < -width) {
 		Lyg = Lxg;
 	} else if (checkwidth <= width) {
@@ -833,24 +828,12 @@ void ZaMultiCompPlugin::run_comp(int k, float in, float *out)
         old_yg[k] = Lyg;
 }
 
-void ZaMultiCompPlugin::pushsample(float samples[], float sample, int k)
+void ZaMultiCompPlugin::pushsample(float sample, int k)
 {
-	++pos[k];
-	if (pos[k] >= MAX_SAMPLES)
-		pos[k] = 0;
-	samples[pos[k]] = sample;
-}
+	const float rate = 2. * M_PI * 5.;
+	float lpf = rate / (rate + getSampleRate());
 
-float ZaMultiCompPlugin::averageabs(float samples[])
-{
-	int i;
-	float average = 0.f;
-
-	for (i = 0; i < MAX_SAMPLES; i++) {
-		average += samples[i]*samples[i];
-	}
-	average /= (float) MAX_SAMPLES;
-	return sqrt(average);
+	average[k] += lpf * (sample*sample - average[k]);
 }
 
 void ZaMultiCompPlugin::run(const float** inputs, float** outputs, uint32_t frames)
@@ -881,7 +864,7 @@ void ZaMultiCompPlugin::run(const float** inputs, float** outputs, uint32_t fram
 		float fil1[2], fil2[2], fil3[2], fil4[2];
 		float outL[MAX_COMP+1] = {0.f};
 		float inl = sanitize_denormal(inputs[0][i]);
-		inl = (fabs(inl) < DANGER) ? inl : 0.f;
+		inl = (fabsf(inl) < DANGER) ? inl : 0.f;
 
 		int listenmode = 0;
 
@@ -889,24 +872,24 @@ void ZaMultiCompPlugin::run(const float** inputs, float** outputs, uint32_t fram
                 run_lr4(0, inl, &fil1[0], &fil2[0]);
                 run_lr4(1, fil2[0], &fil3[0], &fil4[0]);
 
-		pushsample(outlevelold[0], fil1[0], 0);
-		outlevel[0] = averageabs(outlevelold[0]);
+		pushsample(fil1[0], 0);
+		outlevel[0] = sqrt(average[0]);
 		outlevel[0] = (outlevel[0] == 0.f) ? -45.0 : to_dB(outlevel[0]);
 		if (tog1)
 			run_comp(0, fil1[0], &outL[0]);
 
 		tmp1[0] = tog1 ? outL[0] * from_dB(makeup[0]) : fil1[0];
 
-		pushsample(outlevelold[1], fil3[0], 1);
-		outlevel[1] = averageabs(outlevelold[1]);
+		pushsample(fil3[0], 1);
+		outlevel[1] = sqrt(average[1]);
 		outlevel[1] = (outlevel[1] == 0.f) ? -45.0 : to_dB(outlevel[1]);
 		if (tog2)
 			run_comp(1, fil3[0], &outL[1]);
 
                 tmp2[0] = tog2 ? outL[1] * from_dB(makeup[1]) : fil3[0];
 
-		pushsample(outlevelold[2], fil4[0], 2);
-		outlevel[2] = averageabs(outlevelold[2]);
+		pushsample(fil4[0], 2);
+		outlevel[2] = sqrt(average[2]);
 		outlevel[2] = (outlevel[2] == 0.f) ? -45.0 : to_dB(outlevel[2]);
 		if (tog3)
 			run_comp(2, fil4[0], &outL[2]);
