@@ -5,7 +5,7 @@
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of
- * the License, or any later version.
+ * the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -731,11 +731,6 @@ void ZaMultiCompX2Plugin::activate()
 		}
         }
 	maxL = maxR = 0.f;
-	for (i = 0; i < MAX_SAMPLES; i++) {
-		outlevelold[0][i] = 0.f;
-		outlevelold[1][i] = 0.f;
-		outlevelold[2][i] = 0.f;
-	}
 	pos[0] = 0;
 	pos[1] = 0;
 	pos[2] = 0;
@@ -772,7 +767,7 @@ float ZaMultiCompX2Plugin::run_linear_svf_xover(struct linear_svf *self, float i
 	double s1 = self->s[1];
 	double g2 = g*g;
 	double vhigh = in * mixhigh;
-	double vband = in * 0.5;
+	double vband = in * 0.75;
 	double vlow = in * mixlow;
 
 	v[0] = in;
@@ -788,8 +783,8 @@ void ZaMultiCompX2Plugin::calc_lr4(float f, int i)
 {
 	float srate = getSampleRate();
 
-	linear_svf_set_xover(&simper[0][i], srate, f, 0.5);
-	linear_svf_set_xover(&simper[1][i], srate, f, 0.5);
+	linear_svf_set_xover(&simper[0][i], srate, f, 0.25);
+	linear_svf_set_xover(&simper[1][i], srate, f, 0.25);
 }
 
 void ZaMultiCompX2Plugin::run_lr4(int i, float in, float *outlo, float *outhi)
@@ -817,12 +812,12 @@ void ZaMultiCompX2Plugin::run_comp(int k, float inL, float inR, float *outL, flo
         Lyg = Ryg = 0.f;
 	inL = sanitize_denormal(inL);
 	inR = sanitize_denormal(inR);
-        Lxg = (inL==0.f) ? -160.f : to_dB(fabs(inL));
-        Rxg = (inR==0.f) ? -160.f : to_dB(fabs(inR));
+        Lxg = (inL==0.f) ? -160.f : to_dB(fabsf(inL));
+        Rxg = (inR==0.f) ? -160.f : to_dB(fabsf(inR));
         Lxg = sanitize_denormal(Lxg);
         Rxg = sanitize_denormal(Rxg);
 
-	checkwidth = 2.f*fabs(Lxg-thresdb[k]);
+	checkwidth = 2.f*fabsf(Lxg-thresdb[k]);
 	if (2.f*(Lxg-thresdb[k]) < -width) {
 		Lyg = Lxg;
 	} else if (checkwidth <= width) {
@@ -833,7 +828,7 @@ void ZaMultiCompX2Plugin::run_comp(int k, float inL, float inR, float *outL, flo
 		Lyg = sanitize_denormal(Lyg);
 	}
 
-	checkwidth = 2.f*fabs(Rxg-thresdb[k]);
+	checkwidth = 2.f*fabsf(Rxg-thresdb[k]);
 	if (2.f*(Rxg-thresdb[k]) < -width) {
 		Ryg = Rxg;
 	} else if (checkwidth <= width) {
@@ -892,24 +887,12 @@ void ZaMultiCompX2Plugin::run_comp(int k, float inL, float inR, float *outL, flo
         old_yg[1][k] = Ryg;
 }
 
-void ZaMultiCompX2Plugin::pushsample(float samples[], float sample, int k)
+void ZaMultiCompX2Plugin::pushsample(float sample, int k)
 {
-	++pos[k];
-	if (pos[k] >= MAX_SAMPLES)
-		pos[k] = 0;
-	samples[pos[k]] = sample;
-}
+	const float rate = 2. * M_PI * 5.;
+	float lpf = rate / (rate + getSampleRate());
 
-float ZaMultiCompX2Plugin::averageabs(float samples[])
-{
-	int i;
-	float average = 0.f;
-
-	for (i = 0; i < MAX_SAMPLES; i++) {
-		average += samples[i]*samples[i];
-	}
-	average /= (float) MAX_SAMPLES;
-	return sqrt(average);
+	average[k] += lpf * (sample*sample - average[k]);
 }
 
 void ZaMultiCompX2Plugin::run(const float** inputs, float** outputs, uint32_t frames)
@@ -945,8 +928,8 @@ void ZaMultiCompX2Plugin::run(const float** inputs, float** outputs, uint32_t fr
 		float outR[MAX_COMP+1] = {0.f};
 		float inl = sanitize_denormal(inputs[0][i]);
 		float inr = sanitize_denormal(inputs[1][i]);
-		inl = (fabs(inl) < DANGER) ? inl : 0.f;
-		inr = (fabs(inr) < DANGER) ? inr : 0.f;
+		inl = (fabsf(inl) < DANGER) ? inl : 0.f;
+		inr = (fabsf(inr) < DANGER) ? inr : 0.f;
 
 		int listenmode = 0;
 
@@ -956,8 +939,8 @@ void ZaMultiCompX2Plugin::run(const float** inputs, float** outputs, uint32_t fr
                 run_lr4(2, fil2[0], &fil3[0], &fil4[0]);
                 run_lr4(3, fil2[1], &fil3[1], &fil4[1]);
 
-		pushsample(outlevelold[0], std::max(fil1[0], fil1[1]), 0);
-		outlevel[0] = averageabs(outlevelold[0]);
+		pushsample(std::max(fil1[0], fil1[1]), 0);
+		outlevel[0] = sqrt(average[0]);
 		outlevel[0] = (outlevel[0] == 0.f) ? -45.0 : to_dB(outlevel[0]);
 		if (tog1)
 			run_comp(0, fil1[0], fil1[1], &outL[0], &outR[0]);
@@ -965,8 +948,8 @@ void ZaMultiCompX2Plugin::run(const float** inputs, float** outputs, uint32_t fr
 		tmp1[0] = tog1 ? outL[0] * from_dB(makeup[0]) : fil1[0];
                 tmp1[1] = tog1 ? outR[0] * from_dB(makeup[0]) : fil1[1];
 
-		pushsample(outlevelold[1], std::max(fil3[0], fil3[1]), 1);
-		outlevel[1] = averageabs(outlevelold[1]);
+		pushsample(std::max(fil3[0], fil3[1]), 1);
+		outlevel[1] = sqrt(average[1]);
 		outlevel[1] = (outlevel[1] == 0.f) ? -45.0 : to_dB(outlevel[1]);
 		if (tog2)
 			run_comp(1, fil3[0], fil3[1], &outL[1], &outR[1]);
@@ -974,8 +957,8 @@ void ZaMultiCompX2Plugin::run(const float** inputs, float** outputs, uint32_t fr
                 tmp2[0] = tog2 ? outL[1] * from_dB(makeup[1]) : fil3[0];
                 tmp2[1] = tog2 ? outR[1] * from_dB(makeup[1]) : fil3[1];
 
-		pushsample(outlevelold[2], std::max(fil4[0], fil4[1]), 2);
-		outlevel[2] = averageabs(outlevelold[2]);
+		pushsample(std::max(fil4[0], fil4[1]), 2);
+		outlevel[2] = sqrt(average[2]);
 		outlevel[2] = (outlevel[2] == 0.f) ? -45.0 : to_dB(outlevel[2]);
 		if (tog3)
 			run_comp(2, fil4[0], fil4[1], &outL[2], &outR[2]);
