@@ -1,11 +1,11 @@
 /*
- * ZamComp
+ * ZamDelay
  * Copyright (C) 2014 Damien Zammit <damien@zamaudio.com>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of
- * the License, or any later version.
+ * the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -34,5 +34,7 @@
 #define DISTRHO_PLUGIN_IS_RT_SAFE    1
 
 #define DISTRHO_PLUGIN_URI "urn:zamaudio:ZamDelay"
+#define DISTRHO_PLUGIN_LV2_CATEGORY "lv2:DelayPlugin"
+
 
 #endif // DISTRHO_PLUGIN_INFO_H_INCLUDED
