@@ -5,7 +5,7 @@
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of
- * the License, or any later version.
+ * the License, or (at your option) any later version.
  *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -139,8 +139,7 @@ ZamEQ2UI::ZamEQ2UI()
     fSliderMaster->setStartPos(masterPosStart);
     fSliderMaster->setEndPos(masterPosEnd);
     fSliderMaster->setRange(-12.f,12.f);
-    fSliderMaster->setValue(0.f);
-    fSliderMaster->setStep(6.f);
+    fSliderMaster->setDefault(0.f);
     fSliderMaster->setCallback(this);
 
     fCanvasArea.setPos(165,10);
